import pathlib
from configparser import ConfigParser
from typing import Any, Generator

import pytest

import nielsen.config


@pytest.fixture(autouse=True)
def config() -> Generator[ConfigParser, Any, Any]:
    """Fixture to load Nielsen configuration for tests."""

    nielsen.config.load_config(pathlib.Path("./fixtures/config.ini"))
    yield nielsen.config.config
    # Clear all options between uses
    nielsen.config.config.clear()


@pytest.fixture
def missing_file() -> pathlib.Path:
    return pathlib.Path("fixtures/media/missing.file")



"""Test the nielsen.media.TV class."""

import logging
import pathlib
from configparser import ConfigParser
from typing import Any, Callable, Pattern, TypedDict

import pytest
from pytest_mock import MockerFixture, MockType

import nielsen.config
import nielsen.media


class EpisodeMetadata(TypedDict):
    """A simple way to define the structure of episode metadata used in the majority of
    tests."""

    series: str
    season: int
    episode: int
    title: str


# Not quite a fixture, but a list of of tuples containing a filename and a dictionary of
# the metadata that should be inferred by a successful call to the
# nielsen.media.TV.infer method. This can be passed to pytest parametrized test
# functions to avoid building huge lists in the decorator.
EPISODES: list[tuple[str, EpisodeMetadata]] = [
    (
        # Very nicely formatted
        "The.Glades.S02E01.Family.Matters.HDTV.XviD-FQM.avi",
        {
            "series": "The Glades",
            "season": 2,
            "episode": 1,
            "title": "Family Matters",
        },
    ),
    (
        # Needs title casing
        "the.glades.s02e01.family.matters.hdtv.xvid-fqm.avi",
        {
            "series": "The Glades",
            "season": 2,
            "episode": 1,
            "title": "Family Matters",
        },
    ),
    (
        # Missing title
        "The.Glades.S02E01.HDTV.XviD-FQM.avi",
        {
            "series": "The Glades",
            "season": 2,
            "episode": 1,
            "title": "",
        },
    ),
    (
        # Already processed by nielsen
        "The Glades -02.01- Family Matters.avi",
        {
            "series": "The Glades",
            "season": 2,
            "episode": 1,
            "title": "Family Matters",
        },
    ),
    (
        # Another common post-processing format
        "The Glades -201- Family Matters.avi",
        {
            "series": "The Glades",
            "season": 2,
            "episode": 1,
            "title": "Family Matters",
        },
    ),
    (
        # Four digit season/episode code, fewer hyphens
        "Firefly 0101 - Serenity.mkv",
        {
            "series": "Firefly",
            "season": 1,
            "episode": 1,
            "title": "Serenity",
        },
    ),
    (
        # Has most necessary information, but formatted strangely
        "Supernatural.S10E15.mp4",
        {
            "series": "Supernatural",
            "season": 10,
            "episode": 15,
            "title": "",
        },
    ),
    (
        # Same as above, but with an extra dot between season and episode
        "Pushing.Daisies.S02.E03.mp4",
        {
            "series": "Pushing Daisies",
            "season": 2,
            "episode": 3,
            "title": "",
        },
    ),
    (
        # Nicely formatted, but with an apostrophe in the title
        "Person.of.Interest.S0310.The.Devil's.Share.HDTV.avi",
        {
            "series": "Person of Interest",
            "season": 3,
            "episode": 10,
            "title": "The Devil's Share",
        },
    ),
    (
        # Ensure title casing with the apostrophe works well
        "person.of.interest.s03e10.the.devil's.share.hdtv.avi",
        {
            "series": "Person of Interest",
            "season": 3,
            "episode": 10,
            "title": "The Devil's Share",
        },
    ),
    (
        # Testing WEB-RiP tag and series filtering to drop the year
        "Castle.(2009).S07E18.At.Close.Range.720p.WEB-RiP.mp4",
        {
            "series": "Castle",
            "season": 7,
            "episode": 18,
            "title": "At Close Range",
        },
    ),
    (
        # Same as above, but with title casing
        "Castle.(2009).S01E01.Flowers.for.Your.Grave.720p.WEB-RiP.mp4",
        {
            "series": "Castle",
            "season": 1,
            "episode": 1,
            "title": "Flowers For Your Grave",
        },
    ),
    (
        # Four digit season and episode combination
        "supernatural.1117.red.meat.hdtv-lol[ettv].mp4",
        {
            "series": "Supernatural",
            "season": 11,
            "episode": 17,
            "title": "Red Meat",
        },
    ),
    (
        # Specifying file within a directory
        "supernatural.1117.hdtv-lol[ettv]/supernatural.1117.red.meat.hdtv-lol[ettv].mp4",
        {
            "series": "Supernatural",
            "season": 11,
            "episode": 17,
            "title": "Red Meat",
        },
    ),
    (
        # Four digit year with parenthesis followed by three digit season and episode combination
        "the.flash.(2014).217.flash.back.hdtv-lol[ettv].mp4",
        {
            "series": "The Flash",
            "season": 2,
            "episode": 17,
            "title": "Flash Back",
        },
    ),
    (
        # Four digit year with season and episode markers
        "The.Flash.2014.S02E17.Flash.Back.HDTV.x264-LOL[ettv].mp4",
        {
            "series": "The Flash",
            "season": 2,
            "episode": 17,
            "title": "Flash Back",
        },
    ),
    (
        # Four digit year followed by three digit season and episode combination
        "The.Flash.2014.217.Flash.Back.HDTV.x264-LOL[ettv].mp4",
        {
            "series": "The Flash",
            "season": 2,
            "episode": 17,
            "title": "Flash Back",
        },
    ),
    (
        # Tag removal
        "Game.of.Thrones.S06E07.The.Broken.Man.1080p.HDTV.6CH.ShAaNiG.mkv",
        {
            "series": "Game of Thrones",
            "season": 6,
            "episode": 7,
            "title": "The Broken Man",
        },
    ),
    (
        # File in a subdirectory and tag removal
        "Game.of.Thrones.S07E01.720p.HDTV.x264-AVS[rarbg]/Game.of.Thrones.S07E01.Dragonstone.720p.HDTV.x264-AVS.mkv",
        {
            "series": "Game of Thrones",
            "season": 7,
            "episode": 1,
            "title": "Dragonstone",
        },
    ),
    (
        # Single episode with what looks like a second episode marker in the title
        "Sample.Show.S01E01.E19.Protocol.720p.HDTV.X264-DIMENSION.mkv",
        {
            "series": "Sample Show",
            "season": 1,
            "episode": 1,
            "title": "E19 Protocol",
        },
    ),
    (
        # Unusual filename for last ditch effort pattern
        "Limitless S01E11 This Is Your Brian on Drugs (1080p x265 Joy).mkv",
        {
            "series": "Limitless",
            "season": 1,
            "episode": 11,
            "title": "This Is Your Brian On Drugs",
        },
    ),
    (
        # Filename with lots of numbers
        "What.If.2021.S02E08.What.if.the.Avengers.Assembled.in.1602.1080p.DSNP.WEB-DL.DDP5.1.Atmos.H.264-FLUX.mkv",
        {
            "series": "What If",
            "season": 2,
            "episode": 8,
            "title": "What If The Avengers Assembled In 1602",
        },
    ),
]


@pytest.fixture
def tv_good_filename() -> nielsen.media.TV:
    return nielsen.media.TV(
        pathlib.Path("fixtures/tv/Ted Lasso -01.03- Trent Crimm: The Independent.mkv")
    )


@pytest.fixture
def tv_good_metadata() -> nielsen.media.TV:
    return nielsen.media.TV(
        pathlib.Path("fixtures/tv/tv.mkv"),
        series="Ted Lasso",
        season=1,
        episode=3,
        title="Trent Crimm: The Independent",
    )


@pytest.fixture
def tv_good_metadata_missing_file(missing_file) -> nielsen.media.TV:
    return nielsen.media.TV(
        missing_file,
        series="Ted Lasso",
        season=1,
        episode=3,
        title="Trent Crimm: The Independent",
    )


@pytest.fixture
def tv_all_data() -> nielsen.media.Media:
    return nielsen.media.TV(
        pathlib.Path("fixtures/tv/Ted Lasso -01.03- Trent Crimm: The Independent.mkv"),
        series="Ted Lasso",
        season=1,
        episode=3,
        title="Trent Crimm: The Independent",
    )


@pytest.fixture
def tv_no_metadata(mocker) -> Callable[[str], nielsen.media.TV]:
    """Return a mock TV object using the given filename as a Path but setting no
    metadata. Unlike a real TV object, the provided filename need not exist, nor be a
    regular file - those checks are disabled when creating this object."""

    def __tv_factory(filename: str) -> nielsen.media.TV:
        return nielsen.media.TV(pathlib.Path(filename))

    mock_exists: MockType = mocker.patch("pathlib.Path.exists")
    mock_exists.return_value = True
    mock_is_file: MockType = mocker.patch("pathlib.Path.is_file")
    mock_is_file.return_value = True

    return __tv_factory


@pytest.fixture
def tv_factory(mocker) -> Callable[[str, EpisodeMetadata], nielsen.media.TV]:
    """Return a mock TV object using the given filename as a Path and setting any other
    metadata provided. Unlike a real TV object, the provided filename need not exist,
    nor be a regular file - those checks are disabled when creating this object."""

    def __tv_factory(filename: str, metadata: EpisodeMetadata) -> nielsen.media.TV:
        return nielsen.media.TV(
            pathlib.Path(filename),
            series=metadata.get("series", ""),
            season=metadata.get("season", 0),
            episode=metadata.get("episode", 0),
            title=metadata.get("title", ""),
        )

    mock_exists: MockType = mocker.patch("pathlib.Path.exists")
    mock_exists.return_value = True
    mock_is_file: MockType = mocker.patch("pathlib.Path.is_file")
    mock_is_file.return_value = True

    return __tv_factory


@pytest.mark.parametrize(
    "fixt",
    [
        pytest.param("tv_good_metadata", id="good metadata"),
        pytest.param("tv_good_metadata_missing_file", id="good metadata missing file"),
        pytest.param("tv_all_data", id="all data"),
    ],
)
def test_get_metadata(fixt, request) -> None:
    """Return a dictionary with all the relevant fields."""

    tv: nielsen.media.TV = request.getfixturevalue(fixt)
    assert tv.metadata == {
        "series": "Ted Lasso",
        "season": 1,
        "episode": 3,
        "title": "Trent Crimm: The Independent",
    }


def test_get_orgdir(tv_all_data) -> None:
    """The orgdir should be the library, series name, then season."""

    assert (
        tv_all_data.orgdir == pathlib.Path("fixtures/tv/Ted Lasso/Season 01/").resolve()
    )


@pytest.mark.parametrize(
    "fixt",
    [
        pytest.param("tv_good_filename", id="good filename"),
        pytest.param("tv_good_metadata", id="good metadata"),
        pytest.param("tv_good_metadata_missing_file", id="good metadata missing file"),
        pytest.param("tv_all_data", id="all data"),
    ],
)
def test_get_patterns(fixt, request) -> None:
    """TV objects should have a list of patterns."""

    tv: nielsen.media.TV = request.getfixturevalue(fixt)
    assert isinstance(tv.patterns, list)
    for pattern in tv.patterns:
        assert isinstance(pattern, Pattern)


@pytest.mark.parametrize(
    "fixt",
    [
        pytest.param("tv_good_filename", id="good filename"),
        pytest.param("tv_good_metadata", id="good metadata"),
        pytest.param("tv_good_metadata_missing_file", id="good metadata missing file"),
        pytest.param("tv_all_data", id="all data"),
    ],
)
def test_get_section(fixt, request) -> None:
    """The section property should return a value based on the type."""

    tv: nielsen.media.TV = request.getfixturevalue(fixt)
    assert tv.section == "tv", "The section should match the type name, but lowercase"


@pytest.mark.parametrize("filename, metadata", EPISODES)
def test_infer(filename: str, metadata: EpisodeMetadata, tv_no_metadata) -> None:
    """Test every pattern and difficult edge cases."""

    tv: nielsen.media.TV = tv_no_metadata(filename)
    tv.infer()
    assert tv.metadata == metadata, "Inferred metadata mismatch"


def test_ordering(missing_file) -> None:
    """Items should be sorted by season, then episode number."""

    assert nielsen.media.TV(missing_file, season=1, episode=1) < nielsen.media.TV(
        missing_file, season=1, episode=2
    ), "Same season, different episode numbers"

    assert nielsen.media.TV(missing_file, season=1, episode=1) < nielsen.media.TV(
        missing_file, season=2, episode=1
    ), "Different seasons, same episode number"

    assert nielsen.media.TV(missing_file, season=1, episode=10) < nielsen.media.TV(
        missing_file, season=2, episode=2
    ), "Different seasons and episode numbers"

    assert nielsen.media.TV(missing_file, season=1, episode=2) == nielsen.media.TV(
        missing_file, season=1, episode=2
    ), "Same season and episode number"


def test_rename_file_not_found(tv_good_metadata_missing_file) -> None:
    """Rename a TV object with no path or an invalid path."""

    with pytest.raises(FileNotFoundError):
        tv_good_metadata_missing_file.rename()


@pytest.mark.parametrize("simulate", ["true", "false"], ids=["simulate", "no-simulate"])
def test_rename_success(
    simulate: str,
    tv_good_metadata: nielsen.media.TV,
    config: ConfigParser,
) -> None:
    """Successfully rename a file without moving it to a different directory."""

    # Set simulation, which controls whether the file is actually renamed, or the the
    # new path is returned without renaming
    config.set("nielsen", "simulate", simulate)

    # Ensure the source exists and the destination does not
    tv_good_metadata.path.touch(exist_ok=True)
    source: pathlib.Path = tv_good_metadata.path.resolve()
    dest: pathlib.Path = tv_good_metadata.path.with_stem(str(tv_good_metadata))
    dest.unlink(missing_ok=True)
    assert source.exists()
    assert not dest.exists()

    new_path: pathlib.Path = tv_good_metadata.rename()

    if config.getboolean("nielsen", "simulate"):
        assert new_path == source
        assert source.exists()
        assert not dest.exists()
        source.unlink()
    else:
        assert new_path == dest
        assert not source.exists()
        assert dest.exists()
        dest.unlink()


def test_rename_file_exists(
    tv_good_metadata: nielsen.media.TV,
    mocker: MockerFixture,
    caplog: pytest.LogCaptureFixture,
) -> None:
    """Rename a TV object where the destination already exists."""

    caplog.set_level(logging.INFO)

    mock_exists: MockType = mocker.patch("pathlib.Path.exists")
    mock_exists.return_value = True

    mock_samefile: MockType = mocker.patch("pathlib.Path.samefile")
    mock_samefile.return_value = False

    tv_good_metadata.rename()
    assert "FILE_CONFLICT" in caplog.text

    mock_samefile.return_value = True
    tv_good_metadata.rename()
    assert "File already named correctly." in caplog.text


def test_organize_success(tv_factory, mocker: MockerFixture) -> None:
    """Organize file, set and return new path."""

    # Mock the existence of the file on disk to avoid creating and removing
    # files on every test.
    mock_is_file: MockType = mocker.patch("pathlib.Path.is_file")
    mock_is_file.return_value = True

    mock_chmod: MockType = mocker.patch("pathlib.Path.chmod")
    mock_chown: MockType = mocker.patch("nielsen.media.chown")
    mock_move: MockType = mocker.patch("nielsen.media.move")

    # shutil.move moves a file and returns the destination path passed as an
    # argument. Mock it by just returning the input argument.
    mock_move.side_effect = lambda _, org: org
    filename: str = "fixtures/tv/Ted Lasso -01.03- Trent Crimm: The Independent.mkv"
    destination: str = "fixtures/tv/Ted Lasso/Season 01/Ted Lasso -01.03- Trent Crimm: The Independent.mkv"

    tv: nielsen.media.TV = tv_factory(
        filename,
        {
            "series": "Ted Lasso",
            "season": 1,
            "episode": 3,
            "title": "Trent Crimm: The Independent",
        },
    )

    assert tv.organize() == (pathlib.Path(destination).resolve())

    # The pathlib.Path.chmod and shutil.chown functions can be assumed to work properly,
    # we just need to assert that they were called with the correct values.
    mock_chmod.assert_called_with(0o644)
    mock_chown.assert_called_with(tv.path, "nielsen_user", "nielsen_group")


def test_repr(tv_all_data) -> None:
    """Object representation should contain enough information to recreate an object."""

    path: pathlib.Path = pathlib.Path(
        "fixtures/tv/Ted Lasso -01.03- Trent Crimm: The Independent.mkv"
    ).resolve()
    series: str = "Ted Lasso"
    season: int = 1
    episode: int = 3
    title: str = "Trent Crimm: The Independent"
    expected: str = f"<TV(self.{path=}, self.{series=}, self.{season=}, self.{episode=}, self.{title=})>"

    assert repr(tv_all_data) == expected


def test_set_metadata(tv_no_metadata) -> None:
    """Set the metadata dictionary (with type conversions)."""

    metadata: dict[str, Any] = {
        "series": "Ted Lasso",
        "season": "1",
        "episode": "3",
        "title": "Trent Crimm: The Independent",
    }
    tv_no_metadata.metadata = metadata

    assert tv_no_metadata.metadata == metadata


def test_str(tv_good_metadata: nielsen.media.TV) -> None:
    """The string representation should provide a useful display name."""

    expected: str = "Ted Lasso -01.03- Trent Crimm: The Independent"

    assert (
        str(tv_good_metadata) == expected
    ), "String representation should match display format"


def test_str_no_metadata(tv_no_metadata: Callable[[str], nielsen.media.TV]) -> None:
    """The string representation should indicate a lack of metadata."""

    expected: str = "Unknown -00.00- Unknown"

    assert (
        str(tv_no_metadata("Ted Lasso -01.03- Trent Crimm: The Independent.mkv"))
        == expected
    ), "TV object with no metadata, but good filename"


def test_transform_no_section(
    tv_all_data: nielsen.media.TV,
    config: ConfigParser,
    caplog: pytest.LogCaptureFixture,
) -> None:
    """Log a warning and return the input if no config section found."""

    config.clear()

    transformed: str = tv_all_data.transform("series")
    assert "NO_TRANSFORM_SECTION" in caplog.text
    assert tv_all_data.series == transformed


def test_transform_no_option(
    tv_all_data: nielsen.media.TV,
    config: ConfigParser,
    caplog: pytest.LogCaptureFixture,
):
    """Log a warning and return the input if no config option found."""

    config.clear()
    config.add_section("tv/transform/series")
    transformed: str = tv_all_data.transform("series")

    assert "NO_TRANSFORM_OPTION" in caplog.text
    assert tv_all_data.series == transformed


@pytest.mark.parametrize(
    "variant",
    [
        "Marvel's Agents of S.H.I.E.L.D.",
        "Marvel's Agents of S H I E L D ",
        "Marvel's Agents of SHIELD",
        "Agents of S.H.I.E.L.D.",
        "Agents of S H I E L D ",
        "Agents of SHIELD",
    ],
)
def test_transform(
    variant: str,
    tv_factory: Callable[[str, EpisodeMetadata], nielsen.media.TV],
    config: ConfigParser,
    mocker: MockerFixture,
) -> None:
    """Transform series names based on values from the tv/series/transform config section."""

    # Marvel's Agents of S.H.I.E.L.D. has many variants in the way the series name
    # might be listed. Which variant you prefer is a matter of personal preference
    # and can be configured, but once configured they should all resolve to the same
    # user-preferred variant.

    config.clear()
    config.add_section("tv/transform/series")
    expected: str = "Agents of SHIELD"
    config.set("tv/transform/series", variant, expected)

    # Mock is_dir to ensure the library setter succeeds even when we don't know what the
    # filesystem tree actually looks like in automated test environments.
    mock_is_dir: MockType = mocker.patch("pathlib.Path.is_dir")
    mock_is_dir.return_value = True

    shield: nielsen.media.Media = tv_factory(
        "shield.mkv", {"series": variant, "season": 1, "episode": 1, "title": ""}
    )

    assert shield.transform("series") == expected



"""Test the nielsen.config module."""

import pathlib

import pytest
from pytest_mock import MockType

import nielsen.config


@pytest.fixture
def config_file() -> pathlib.Path:
    """Return a Path object representing the location of the configuration file to be
    used with tests."""

    return pathlib.Path("fixtures/config.ini")


def test_load_config_no_arg_no_files(mocker):
    """Load only the default options."""

    # Mock the default files to avoid polluting the test with user configurations.
    mocker.patch("nielsen.config.CONFIG_FILE_LOCATIONS", new=[])
    assert nielsen.config.load_config() == [], "No files should be loaded."

    for option, value in nielsen.config.config.defaults().items():
        assert (
            nielsen.config.config.get("nielsen", option) == value
        ), "Arbitrary section should return all default values."


def test_load_config_specific_file(config_file):
    """Load config from a specific file."""

    files: list[str] = nielsen.config.load_config(config_file)
    assert files == ["fixtures/config.ini"]

    assert nielsen.config.config.has_section(
        "unit tests"
    ), "The config must have the section from the config file fixture."

    assert nielsen.config.config.get("unit tests", "foo") == "bar"


def test_load_config_missing_file(caplog):
    """Specify a missing configuration file to load."""

    file: pathlib.Path = pathlib.Path("fixtures/missing.ini")
    nielsen.config.load_config(file)
    assert f"Failed to load configuration from: {file}" in caplog.text


def test_write_config(mocker):
    """Write the config to a file."""

    # This function just calls the ConfigParser.write() method, so there's not much
    # for us to test. Ensure our function writes data to the specified file.
    file: pathlib.Path = pathlib.Path("fixtures/write-test.ini")
    mock_open: MockType = mocker.patch("builtins.open")
    mock_write: MockType = mocker.patch("configparser.ConfigParser.write")

    nielsen.config.write_config(file)
    mock_open.assert_called_with(file, mode="w")
    mock_write.assert_called_with(mock_open().__enter__())


def test_update_config(config_file, mocker):
    """Call load_config and write_config with the correct arguments."""

    mock_load_config: MockType = mocker.patch("nielsen.config.load_config")
    mock_write_config: MockType = mocker.patch("nielsen.config.write_config")
    nielsen.config.update_config(config_file)
    mock_load_config.assert_called_with(config_file)
    mock_write_config.assert_called_with(config_file)



#!/usr/bin/env python3
"""Subcommands for interacting with Nielsen's configuration."""

import logging

import nielsen.config
from nielsen.config import config as config

import typer
from rich.pretty import pprint

logger: logging.Logger = logging.getLogger(__name__)
nielsen.config.load_config()

app: typer.Typer = typer.Typer()


@app.command()
def dump() -> None:
    """Dump the current configuration."""

    typer.echo("Configuration")
    typer.echo(config.default_section)
    pprint(config.defaults(), expand_all=True)

    for section in config.sections():
        typer.echo(section)
        pprint(config.items(section))


if __name__ == "__main__":
    app()



from typing import TypedDict


class WebChannel(TypedDict, total=False):
    id: int
    name: str
    country: dict | None
    officialSite: str


class Image(TypedDict, total=False):
    medium: str
    original: str


class Show(TypedDict, total=False):
    id: int
    name: str
    premiered: str | None
    network: dict | None
    webChannel: WebChannel | None
    image: Image | None
    summary: str | None


class Season(TypedDict, total=False):
    id: int
    url: str
    number: int
    name: str
    episodeOrder: int
    premiereDate: str | None
    endDate: str | None
    network: dict | None
    webChannel: WebChannel | None
    image: Image | None
    summary: str | None
    _links: dict



"""Tests for the nielsen.processor module."""

import pathlib

from pytest_mock import MockType

import nielsen.config
import nielsen.media
import nielsen.fetcher
import nielsen.processor


def test_processor_init() -> None:
    """Test construction of new Processor objects."""

    processor: nielsen.processor.Processor = nielsen.processor.Processor(
        nielsen.media.Media, nielsen.fetcher.TVMaze()
    )

    assert hasattr(processor, "media_type")
    assert hasattr(processor, "fetcher")
    assert issubclass(processor.media_type, nielsen.media.Media)
    assert isinstance(processor.fetcher, nielsen.fetcher.TVMaze)


def test_processor_factory_init() -> None:
    """Test construction of new ProcessorFactory objects."""

    factory: nielsen.processor.ProcessorFactory = nielsen.processor.ProcessorFactory(
        nielsen.media.Media, nielsen.fetcher.TVMaze
    )

    assert hasattr(factory, "media_type")
    assert hasattr(factory, "fetcher")
    assert issubclass(factory.media_type, nielsen.media.Media)
    # factory.fetcher must be a subclass of Fetcher, but not an instance of it
    assert issubclass(factory.fetcher, nielsen.fetcher.TVMaze)
    assert not isinstance(factory.fetcher, nielsen.fetcher.TVMaze)


def test_processor_factory_call() -> None:
    """Calling the Factory should return a Processor."""

    factory: nielsen.processor.ProcessorFactory = nielsen.processor.ProcessorFactory(
        nielsen.media.TV, nielsen.fetcher.TVMaze
    )

    processor: nielsen.processor.Processor = factory()
    assert isinstance(processor, nielsen.processor.Processor)


def test_processor_process(mocker) -> None:
    """The process method should call all other high-level Media/Fetcher functions."""

    # This test should be expanded as more Media and Fetcher types are added
    mock_fetcher: MockType = mocker.Mock(spec_set=nielsen.fetcher.TVMaze)
    mock_rename: MockType = mocker.patch("nielsen.media.Media.rename")
    mock_organize: MockType = mocker.patch("nielsen.media.Media.organize")

    media_path: pathlib.Path = pathlib.Path("ted.lasso.s01e01.1080p.web.h264-ggwp")

    processor: nielsen.processor.Processor = nielsen.processor.Processor(
        nielsen.media.TV, mock_fetcher
    )

    # Assert all options are enabled
    assert nielsen.config.config.getboolean("tv", "fetch")
    assert nielsen.config.config.getboolean("tv", "rename")
    assert nielsen.config.config.getboolean("tv", "organize")

    # Process media
    processed: nielsen.media.Media = processor.process(media_path)

    # Assert functions are called
    mock_fetcher.fetch.assert_called_with(processed)
    mock_rename.assert_called_once()
    mock_organize.assert_called_once()

    # Assert returned type of processed Media matches
    assert isinstance(processed, nielsen.media.TV)



"""Test the nielsen.media.Media base class."""

import pathlib
import re
from typing import Any

import pytest
from pytest_mock import MockType

import nielsen.config
import nielsen.media


@pytest.fixture
def good_path() -> nielsen.media.Media:
    """Return a Media object with a valid file path."""

    return nielsen.media.Media(pathlib.Path("fixtures/media.file"))


@pytest.fixture
def missing_file() -> nielsen.media.Media:
    """Return a Media object with no file at the specified path."""

    return nielsen.media.Media(pathlib.Path("fixtures/media/missing.file"))


@pytest.fixture
def non_file_path() -> nielsen.media.Media:
    """Return a Media object with a non-file path."""

    return nielsen.media.Media(pathlib.Path("/dev/null"))


@pytest.fixture
def medias(good_path, missing_file, non_file_path) -> list[nielsen.media.Media]:
    return [
        good_path,
        missing_file,
        non_file_path,
    ]


def test_init(good_path) -> None:
    """Test construction of new Media objects."""

    invalid_paths: list[Any] = [None, False, 0, ""]
    for item in invalid_paths:
        with pytest.raises(TypeError):
            nielsen.media.Media(item)

    assert good_path.section == "media", "Section attribute based on type."

    # The library attribute must be a path, but the default value isn't especially
    # important for generic Media objects.
    assert isinstance(
        good_path.library, pathlib.Path
    ), "Library attribute must be a Path."

    assert good_path.library.is_absolute(), "Library should be an absolute path."


def test_infer_no_patterns(caplog, good_path) -> None:
    """Cannot infer information about an object with no patterns to match."""

    good_path.infer()
    assert "NO_PATTERNS" in caplog.text, "Log an error when inferring without patterns"


def test_match_no_match(good_path) -> None:
    """Return an empty metadata dictionary and log a NO_MATCH message."""

    # Add a pattern just to ensure that the match reaches it and fails to match.
    good_path.patterns = [re.compile(r"USELESS_PATTERN")]

    assert good_path._match() == {}, "Return an empty dictionary."


def test_get_library(good_path) -> None:
    """Get the library property from the appropriate config section."""

    # Calling resolve on the Path we compare to also ensures the library property is
    # always an absolute path.
    assert (
        good_path.library == nielsen.config.config.getpath("media", "library").resolve()  # type: ignore
    ), "Should match option from tv section of config."
    assert (
        good_path.library == pathlib.Path("fixtures/media/").resolve()
    ), "Should match known type-specific value."


def test_get_metadata(good_path) -> None:
    """Method not implemented for base Media class."""

    with pytest.raises(NotImplementedError):
        good_path.metadata


def test_get_orgdir(medias) -> None:
    """Media base objects have no orgdir property."""

    for media in medias:
        with pytest.raises(NotImplementedError):
            media.orgdir


def test_get_path(medias) -> None:
    """Get the path property of the Media object."""

    for media in medias:
        assert isinstance(media.path, pathlib.Path), "Must be a Path object."


def test_get_patterns(medias) -> None:
    """Media base objects have no patterns."""

    for media in medias:
        assert media.patterns == [], "List of patterns must be empty for Media type"


def test_get_section(medias) -> None:
    """The section property should return a value based on the type."""

    for media in medias:
        assert (
            media.section == "media"
        ), "The section should match the type name, but lowercase"


def test_organize_invalid_path(non_file_path) -> None:
    """Media with no path or a non-file path cannot be organized."""

    with pytest.raises(TypeError):
        non_file_path.organize()


def test_organize_library_not_a_directory_error(good_path, mocker) -> None:
    """Library path does not point to a directory."""

    mock_is_file: MockType = mocker.patch("pathlib.Path.is_file")
    mock_is_dir: MockType = mocker.patch("pathlib.Path.is_dir")
    mock_mkdir: MockType = mocker.patch("pathlib.Path.mkdir")

    mock_is_file.return_value = True
    mock_is_dir.return_value = False
    mock_mkdir.side_effect = NotADirectoryError()

    with pytest.raises(NotADirectoryError):
        good_path.organize()


def test_organize_library_permission_error(good_path, mocker) -> None:
    """Library directory does not exist and cannot be created."""

    mock_is_file: MockType = mocker.patch("pathlib.Path.is_file")
    mock_is_dir: MockType = mocker.patch("pathlib.Path.is_dir")
    mock_mkdir: MockType = mocker.patch("pathlib.Path.mkdir")

    mock_is_file.return_value = True
    mock_is_dir.return_value = False
    mock_mkdir.side_effect = PermissionError()

    with pytest.raises(PermissionError):
        good_path.organize()


def test_organize_pass_guards(good_path, mocker) -> None:
    """Pass the guard clauses, but fail because Media has no orgdir."""

    mock_is_file: MockType = mocker.patch("pathlib.Path.is_file")
    mock_move: MockType = mocker.patch("shutil.move")

    # Mock the existence of the file on disk to avoid creating and removing
    # files on every test.
    mock_is_file.return_value = True
    # shutil.move moves a file and returns the destination path passed as an
    # argument. Mock it by just returning the input argument.
    mock_move.return_value = lambda x: x

    assert isinstance(good_path.path, pathlib.Path)

    with pytest.raises(NotImplementedError):
        good_path.organize()


def test_rename_file_not_found(good_path, mocker) -> None:
    """Raise an exception when renaming base Media objects."""

    mock_exists: MockType = mocker.patch("pathlib.Path.exists")
    mock_exists.return_value = False

    with pytest.raises(FileNotFoundError):
        good_path.rename()


def test_str(good_path) -> None:
    """String representation should just be a file path."""

    assert str(good_path) == str(good_path.path.resolve())


def test_set_metadata(good_path) -> None:
    """Method not implemented for base Media class."""

    with pytest.raises(NotImplementedError):
        good_path.metadata = {}


def test_set_section(non_file_path) -> None:
    """Set the section property."""

    assert non_file_path.section == "media", "The section should match the type"

    # Set a new section so the change can be verified
    existing_section: str = "unit tests"
    non_file_path.section = existing_section
    assert (
        non_file_path.section == existing_section
    ), "The section should match the existing section assigned to it"

    new_section: str = "new section"
    assert not nielsen.config.config.has_section(
        new_section
    ), "New section should not yet exist"

    non_file_path.section = new_section
    assert (
        non_file_path.section == new_section
    ), "The section should match the newly created section assigned to it"
    assert nielsen.config.config.has_section(
        new_section
    ), "The new section should be added to the config"


@pytest.mark.parametrize(
    "location",
    [
        pytest.param("fixtures/media/", id="string"),
        pytest.param(pathlib.Path("fixtures/media/"), id="path"),
    ],
)
def test_set_library(good_path, location) -> None:
    """Set the library property to valid paths."""

    good_path.library = location
    assert isinstance(good_path.library, pathlib.Path)
    assert good_path.library.is_dir()


@pytest.mark.parametrize(
    "location",
    [
        pytest.param(None, id="none"),
        pytest.param(0, id="zero"),
        pytest.param(False, id="false"),
        pytest.param("/dev/null", id="non-file string"),
        pytest.param(pathlib.Path("/dev/null"), id="non-file path"),
    ],
)
def test_set_library_invalid(good_path, location) -> None:
    """Set the library property to invalid paths."""

    with pytest.raises(TypeError):
        good_path.library = location


@pytest.mark.parametrize(
    "path",
    [
        pytest.param("fixtures/media.file", id="string"),
        pytest.param(pathlib.Path("fixtures/media.file"), id="path"),
    ],
)
def test_set_path(path) -> None:
    """Set the path property of a Media object."""

    media: nielsen.media.Media = nielsen.media.Media(path)
    assert isinstance(media.path, pathlib.Path)
    assert media.path.is_file()

    with pytest.raises(TypeError):
        media.path = None  # type: ignore



"""Interact with configuration files and objects."""

import logging
import pathlib
from configparser import ConfigParser
from typing import Optional

logger: logging.Logger = logging.getLogger(__name__)
logger.addHandler(logging.NullHandler())

CONFIG_FILE_LOCATIONS: list[str | pathlib.Path] = [
    "/etc/nielsen/config.ini",
    pathlib.Path("~/.config/nielsen/config.ini").expanduser(),
]

# Add a getpath converter.
config: ConfigParser = ConfigParser(
    converters={"path": pathlib.Path}, default_section="nielsen"
)

# Set default options
config[config.default_section] = {
    # Dry Run - Outputs results without actually modifying files
    "simulate": "False",
    # Fetch - Whether to query remote sources for information
    "fetch": "True",
    # Transform - Whether to refer to the <type>/<field>/transform section when organizing Media
    "transform": "True",
    # Interactive - Whether to prompt the user to make decisions while processing
    "interactive": "True",
    # Library - The directory under which Media should be organized
    "library": str(pathlib.Path.home()),
    # LogFile - The file to which log messages should be written
    "logfile": "~/.local/log/nielsen/nielsen.log",
    # LogLevel - The minimum log-level to display
    "loglevel": "WARNING",
    # Mode - The file mode (permissions) to set on Media files during processing
    "mode": "644",
    # Organize - Whether to move Media to its library directory
    "organize": "True",
    # Rename - Whether to rename individual Media files
    "rename": "True",
}


def load_config(path: Optional[pathlib.Path] = None) -> list[str]:
    """Load a configuration from a file into the global configuration object. If no file
    path is provided, default configuration file locations are used. Returns a list of
    files loaded."""

    global config
    files: list[str]

    if not path:
        files = config.read(CONFIG_FILE_LOCATIONS)
        logger.debug("Loaded configuration from default locations: %s", files)
    else:
        files = config.read(str(path))
        if files:
            logger.debug("Loaded configuration from: %s", files)
        else:
            logger.error("Failed to load configuration from: %s", path)

    return files


def write_config(path: pathlib.Path) -> None:
    """Write the global configuration object to the given `path`."""

    global config

    with open(path, mode="w") as file:
        config.write(file)


def update_config(path: pathlib.Path) -> None:
    """Write new data to the config file without changing options already present in the
    config files. By re-reading the config files before writing, runtime options or
    other dynamic changes to the config will be reverted before the file is written."""

    load_config(path.expanduser())
    write_config(path.expanduser())


# vim: tabstop=4 softtabstop=4 shiftwidth=4 expandtab



#!/usr/bin/env python3
"""Command-line frontend for Nielsen."""

import logging
from pathlib import Path
from typing_extensions import Annotated

from nielsen.config import config
import nielsen.config
import nielsen.fetcher
import nielsen.media
import nielsen.processor

import nielsen.bin.cli.config
import nielsen.bin.cli.tv

import typer

config_files: list[str] = nielsen.config.load_config()
logger: logging.Logger = logging.getLogger("nielsen")

logging.basicConfig(
    format="[%(asctime)s][%(levelname)s][%(module)s][%(funcName)s:%(lineno)d]: %(message)s",
    datefmt="%Y-%m-%d %H:%M:%S",
    level=config.get("nielsen", "loglevel", fallback=logging.INFO),
)


app: typer.Typer = typer.Typer(no_args_is_help=True)

app.add_typer(
    nielsen.bin.cli.config.app,
    name="config",
    help="Subcommands for the Nielsen configuration",
    no_args_is_help=True,
)
app.add_typer(
    nielsen.bin.cli.tv.app,
    name="tv",
    help="Subcommands for interacting with TV shows",
    no_args_is_help=True,
)


@app.command()
def infer(
    files: Annotated[list[str], typer.Argument(help="File(s) to process")],
    fetch: Annotated[
        bool,
        typer.Option(help="Fetch metadata from remote sources for file(s)"),
    ] = config.getboolean("nielsen", "fetch"),
    media_type: Annotated[
        nielsen.processor.MediaType,
        typer.Option(help="Media Type used to process files"),
    ] = nielsen.processor.MediaType.TV,
) -> None:
    """Infer metadata about the given file and print it. Does not modify the file."""

    config.set("nielsen", "fetch", str(fetch))

    processor_factory: nielsen.processor.ProcessorFactory | None = (
        nielsen.processor.PROCESSOR_FACTORIES.get(media_type)
    )

    if processor_factory is None:
        logger.critical(
            "Media Processor could not be created for type: %s.", media_type
        )
        raise typer.Exit(code=1)

    processor: nielsen.processor.Processor = processor_factory()

    for file in files:
        media: nielsen.media.Media = processor.media_type(Path(file))
        media.infer()
        processor.fetcher.fetch(media)
        print(media)


@app.command()
def process(
    files: Annotated[list[str], typer.Argument(help="File(s) to process")],
    fetch: Annotated[
        bool,
        typer.Option(help="Fetch metadata from remote sources for file(s)"),
    ] = config.getboolean("nielsen", "fetch"),
    organize: Annotated[
        bool,
        typer.Option(help="Move file(s) to library, but do not otherwise rename them"),
    ] = config.getboolean("nielsen", "organize"),
    rename: Annotated[
        bool, typer.Option(help="Rename file(s), but do not otherwise move them")
    ] = config.getboolean("nielsen", "rename"),
    simulate: Annotated[
        bool,
        typer.Option(help="Show file operations without performing them"),
    ] = config.getboolean("nielsen", "simulate"),
    media_type: Annotated[
        nielsen.processor.MediaType,
        typer.Option(help="Media Type used to process files"),
    ] = nielsen.processor.MediaType.TV,
) -> None:
    """Process the given files. Use the Media Type to determine how files should be
    processed (i.e. which Media subclass and Fetcher to use)."""

    processor_factory: nielsen.processor.ProcessorFactory | None = (
        nielsen.processor.PROCESSOR_FACTORIES.get(media_type)
    )

    if processor_factory is None:
        logger.critical(
            "Media Processor could not be created for type: %s.", media_type
        )
        raise typer.Exit(code=1)

    processor: nielsen.processor.Processor = processor_factory()

    if not config.has_section(media_type.value):
        config.add_section(media_type.value)

    config.set(media_type.value, "fetch", str(fetch))
    config.set(media_type.value, "organize", str(organize))
    config.set(media_type.value, "rename", str(rename))
    config.set("nielsen", "simulate", str(simulate))

    for file in files:
        processor.process(Path(file))

    # Due to the order in which configuration files are loaded, the most "personal"
    # version will be at the end of the list.
    if config_files:
        nielsen.config.update_config(Path(config_files[-1]))


def main() -> None:
    """Rename and organize media files."""

    app()


if __name__ == "__main__":
    typer.run(main)



import pathlib
from configparser import ConfigParser
from typing import Any, Callable

import pytest
from pytest_mock import MockerFixture, MockType
from requests.models import Response

import nielsen.config
import nielsen.fetcher
import nielsen.media


@pytest.fixture
def fetcher() -> nielsen.fetcher.TVMaze:
    """Return a TVMaze fetcher."""

    return nielsen.fetcher.TVMaze()


@pytest.fixture
def ted_lasso_series_id() -> int:
    """Return the TVMaze ID for Ted Lasso to have a consistently available show ID."""

    return 44458


@pytest.fixture
def ted_lasso_season2_id() -> int:
    """Return the TVMaze ID for Ted Lasso season 2 to have a consistently available
    season ID."""

    return 112939


@pytest.fixture
def mock_get(mocker: MockerFixture) -> MockType:
    """Return a mocked version of requests.get."""

    return mocker.patch("requests.get")


@pytest.fixture
def mock_input(mocker: MockerFixture) -> MockType:
    """Return a mocked version of builtins.input."""

    return mocker.patch("builtins.input")


@pytest.fixture
def mock_tv(mocker: MockerFixture) -> MockType:
    """Return a MagicMock specced for a nielsen.media.TV instance."""

    return mocker.MagicMock(spec=nielsen.media.TV)


@pytest.fixture
def response_factory(
    mocker: MockerFixture,
) -> Callable[[str, bool, dict | list[dict]], MockType]:
    """Return a MagicMock specced for a requests.Response instance."""

    def __response_factory(url: str, ok: bool, data: dict | list[dict]) -> MockType:
        mock_response: MockType = mocker.MagicMock(spec=Response)
        mock_response.url = url
        mock_response.ok.return_value = ok
        mock_response.json.return_value = data

        return mock_response

    return __response_factory


@pytest.fixture
def search_shows_agents_of_shield() -> list[dict]:
    """Return the JSON results of https://api.tvmaze.com/search/shows?q=Agents+of+SHIELD"""

    return [
        {
            "score": 1.261306,
            "show": {
                "id": 31,
                "url": "https://www.tvmaze.com/shows/31/marvels-agents-of-shield",
                "name": "Marvel'sAgents of S.H.I.E.L.D.",
                "type": "Scripted",
                "language": "English",
                "genres": ["Action", "Adventure", "Science-Fiction"],
                "status": "Ended",
                "runtime": 60,
                "averageRuntime": 60,
                "premiered": "2013-09-24",
                "ended": "2020-08-12",
                "officialSite": "http://abc.go.com/shows/marvels-agents-of-shield",
                "schedule": {"time": "22:00", "days": ["Wednesday"]},
                "rating": {"average": 8},
                "weight": 97,
                "network": {
                    "id": 3,
                    "name": "ABC",
                    "country": {
                        "name": "United States",
                        "code": "US",
                        "timezone": "America/New_York",
                    },
                    "officialSite": "https://abc.com/",
                },
                "webChannel": None,
                "dvdCountry": None,
                "externals": {"tvrage": 32656, "thetvdb": 263365, "imdb": "tt2364582"},
                "image": {
                    "medium": "https://static.tvmaze.com/uploads/images/medium_portrait/486/1215694.jpg",
                    "original": "https://static.tvmaze.com/uploads/images/original_untouched/486/1215694.jpg",
                },
                "summary": "<p>Phil Coulsonheads an elite team of fellow agents with the worldwide law-enforcement organization known as S.H.I.E.L.D. (Strategic Homeland Intervention Enforcement and Logistics Division), as they investigate strange occurrences around the globe. Its members --each of whom brings a specialty to the group -- work with Coulson to protect those who cannot protect themselves from extraordinary and inconceivable threats.</p>",
                "updated": 1711680992,
                "_links": {
                    "self": {"href": "https://api.tvmaze.com/shows/31"},
                    "previousepisode": {
                        "href": "https://api.tvmaze.com/episodes/1839340",
                        "name": "What We're Fighting For",
                    },
                },
            },
        },
        {
            "score": 1.1468165,
            "show": {
                "id": 23460,
                "url": "https://www.tvmaze.com/shows/23460/marvels-agents-of-shield-slingshot",
                "name": "Marvel's Agents of S.H.I.E.L.D.: Slingshot",
                "type": "Scripted",
                "language": "English",
                "genres": ["Action", "Science-Fiction"],
                "status": "Ended",
                "runtime": None,
                "averageRuntime": 5,
                "premiered": "2016-12-13",
                "ended": "2016-12-13",
                "officialSite": "http://abc.go.com/shows/marvels-agents-of-shield-slingshot",
                "schedule": {"time": "", "days": ["Tuesday"]},
                "rating": {"average": 7.1},
                "weight": 91,
                "network": None,
                "webChannel": {
                    "id": 95,
                    "name": "ABC.com",
                    "country": {
                        "name": "United States",
                        "code": "US",
                        "timezone": "America/New_York",
                    },
                    "officialSite": "https://abc.com/",
                },
                "dvdCountry": None,
                "externals": {"tvrage": None, "thetvdb": 320925, "imdb": "tt6313900"},
                "image": {
                    "medium": "https://static.tvmaze.com/uploads/images/medium_portrait/89/222865.jpg",
                    "original": "https://static.tvmaze.com/uploads/images/original_untouched/89/222865.jpg",
                },
                "summary": "<p><b>Marvel's Agents of S.H.I.E.L.D.: Slingshot</b> is set in the world of the hit television series <i>Marvel's Agents of S.H.I.E.L.D.</i> Taking place shortly before the beginning of Season 4, this digital series features the character of Elena \"Yo-Yo\" Rodriguez, an Inhuman with the ability to move with super-speed. As a person with powers, she must sign the recently instituted Sokovia Accords, the worldwide agreement that regulates and tracks those with super powers. However, the restrictions of the Accords are in direct conflict with a personal mission she's desperate to fulfill, a mission that will test her abilities, her allegiances, and will include some tense encounters with our most popular S.H.I.E.L.D. team members.</p>",
                "updated": 1653922927,
                "_links": {
                    "self": {"href": "https://api.tvmaze.com/shows/23460"},
                    "previousepisode": {
                        "href": "https://api.tvmaze.com/episodes/1014343",
                        "name": "Justicia",
                    },
                },
            },
        },
        {
            "score": 1.1370964,
            "show": {
                "id": 16281,
                "url": "https://www.tvmaze.com/shows/16281/marvels-agents-of-shield-academy",
                "name": "Marvel's Agents of S.H.I.E.L.D.: Academy",
                "type": "Scripted",
                "language": "English",
                "genres": [],
                "status": "Ended",
                "runtime": 5,
                "averageRuntime": 5,
                "premiered": "2016-03-09",
                "ended": "2016-05-04",
                "officialSite": None,
                "schedule": {"time": "", "days": ["Wednesday"]},
                "rating": {"average": 5.9},
                "weight": 73,
                "network": None,
                "webChannel": {
                    "id": 95,
                    "name": "ABC.com",
                    "country": {
                        "name": "United States",
                        "code": "US",
                        "timezone": "America/New_York",
                    },
                    "officialSite": "https://abc.com/",
                },
                "dvdCountry": None,
                "externals": {"tvrage": None, "thetvdb": None, "imdb": "tt6243582"},
                "image": {
                    "medium": "https://static.tvmaze.com/uploads/images/medium_portrait/54/135729.jpg",
                    "original": "https://static.tvmaze.com/uploads/images/original_untouched/54/135729.jpg",
                },
                "summary": "<p>ABC Entertainment and Marvel Television test the physical and mental limits of three super-fans as they compete for the adventure of a lifetime in this new five-part digital original series, <b>Marvel's Agents of S.H.I.E.L.D.: Academy</b>.</p>",
                "updated": 1679255803,
                "_links": {
                    "self": {"href": "https://api.tvmaze.com/shows/16281"},
                    "previousepisode": {
                        "href": "https://api.tvmaze.com/episodes/741268",
                        "name": "Commencement",
                    },
                },
            },
        },
        {
            "score": 1.0545623,
            "show": {
                "id": 16280,
                "url": "https://www.tvmaze.com/shows/16280/marvels-agents-of-shield-double-agent",
                "name": "Marvel's Agents of S.H.I.E.L.D.: Double Agent",
                "type": "Scripted",
                "language": "English",
                "genres": [],
                "status": "Ended",
                "runtime": 5,
                "averageRuntime": 5,
                "premiered": "2015-03-04",
                "ended": "2015-05-06",
                "officialSite": None,
                "schedule": {"time": "", "days": ["Wednesday"]},
                "rating": {"average": 6.8},
                "weight": 73,
                "network": None,
                "webChannel": {
                    "id": 95,
                    "name": "ABC.com",
                    "country": {
                        "name": "United States",
                        "code": "US",
                        "timezone": "America/New_York",
                    },
                    "officialSite": "https://abc.com/",
                },
                "dvdCountry": None,
                "externals": {"tvrage": None, "thetvdb": None, "imdb": "tt4501242"},
                "image": {
                    "medium": "https://static.tvmaze.com/uploads/images/medium_portrait/89/222843.jpg",
                    "original": "https://static.tvmaze.com/uploads/images/original_untouched/89/222843.jpg",
                },
                "summary": "<p>There is a double agent on the set of Marvel's Agents of S.H.I.E.L.D. Reporting to a shadowy Mastermind, he evades security and gathers classified information for the legionsof fans who are eager for the next exciting bit of Marvel news.</p>",
                "updated": 1653922713,
                "_links": {
                    "self": {"href": "https://api.tvmaze.com/shows/16280"},
                    "previousepisode": {
                        "href": "https://api.tvmaze.com/episodes/741258",
                        "name": "The Mastermind Is Revealed",
                    },
                },
            },
        },
    ]


@pytest.fixture
def search_shows_ted_lasso() -> list[dict]:
    """Return the JSON results from https://api.tvmaze.com/search/shows?q=Ted+Lasso"""

    return [
        {
            "score": 1.2008839,
            "show": {
                "id": 44458,
                "url": "https://www.tvmaze.com/shows/44458/ted-lasso",
                "name": "Ted Lasso",
                "type": "Scripted",
                "language": "English",
                "genres": ["Drama", "Comedy", "Sports"],
                "status": "Running",
                "runtime": None,
                "averageRuntime": 42,
                "premiered": "2020-08-14",
                "ended": None,
                "officialSite": "https://tv.apple.com/show/ted-lasso/umc.cmc.vtoh0mn0xn7t3c643xqonfzy",
                "schedule": {"time": "", "days": ["Wednesday"]},
                "rating": {"average": 8.1},
                "weight": 96,
                "network": None,
                "webChannel": {
                    "id": 310,
                    "name": "Apple TV+",
                    "country": None,
                    "officialSite": "https://tv.apple.com/",
                },
                "dvdCountry": None,
                "externals": {"tvrage": None, "thetvdb": 383203, "imdb": "tt10986410"},
                "image": {
                    "medium": "https://static.tvmaze.com/uploads/images/medium_portrait/457/1142533.jpg",
                    "original": "https://static.tvmaze.com/uploads/images/original_untouched/457/1142533.jpg",
                },
                "summary": "<p><b>Ted Lasso </b>centers on an idealistic — and clueless — all-American football coach hired to manage an English football club — despite having no soccer coaching experience at all.</p>",
                "updated": 1724535610,
                "_links": {
                    "self": {"href": "https://api.tvmaze.com/shows/44458"},
                    "previousepisode": {
                        "href": "https://api.tvmaze.com/episodes/2490079",
                        "name": "So Long, Farewell",
                    },
                },
            },
        }
    ]


@pytest.fixture
def search_shows_useless() -> list[dict]:
    """Return the JSON results of https://api.tvmaze.com/search/shows?q=useless+search+string"""

    return []


@pytest.fixture
def seasons_episodes_ted_lasso_s2() -> list[dict]:
    """Return the JSON results of https://api.tvmaze.com/seasons/112939/episodes"""

    return [
        {
            "id": 2075166,
            "url": "https://www.tvmaze.com/episodes/2075166/ted-lasso-2x01-goodbye-earl",
            "name": "Goodbye Earl",
            "season": 2,
            "number": 1,
            "type": "regular",
            "airdate": "2021-07-23",
            "airtime": "",
            "airstamp": "2021-07-23T12:00:00+00:00",
            "runtime": 34,
            "rating": {"average": 7.8},
            "image": {
                "medium": "https://static.tvmaze.com/uploads/images/medium_landscape/341/852868.jpg",
                "original": "https://static.tvmaze.com/uploads/images/original_untouched/341/852868.jpg",
            },
            "summary": "<p>AFC Richmond brings in a sports psychologist to help the team overcome their unprecedented seven game tie-streak.</p>",
            "_links": {
                "self": {"href": "https://api.tvmaze.com/episodes/2075166"},
                "show": {
                    "href": "https://api.tvmaze.com/shows/44458",
                    "name": "TedLasso",
                },
            },
        },
        {
            "id": 2132225,
            "url": "https://www.tvmaze.com/episodes/2132225/ted-lasso-2x02-lavender",
            "name": "Lavender",
            "season": 2,
            "number": 2,
            "type": "regular",
            "airdate": "2021-07-30",
            "airtime": "",
            "airstamp": "2021-07-30T12:00:00+00:00",
            "runtime": 33,
            "rating": {"average": 7.9},
            "image": {
                "medium": "https://static.tvmaze.com/uploads/images/medium_landscape/341/852869.jpg",
                "original": "https://static.tvmaze.com/uploads/images/original_untouched/341/852869.jpg",
            },
            "summary": "<p>Ted is surprised by the reappearance of a familiar face. Roy tries out a new gig.</p>",
            "_links": {
                "self": {"href": "https://api.tvmaze.com/episodes/2132225"},
                "show": {
                    "href": "https://api.tvmaze.com/shows/44458",
                    "name": "Ted Lasso",
                },
            },
        },
        {
            "id": 2138281,
            "url": "https://www.tvmaze.com/episodes/2138281/ted-lasso-2x03-do-the-right-est-thing",
            "name": "Do the Right-est Thing",
            "season": 2,
            "number": 3,
            "type": "regular",
            "airdate": "2021-08-06",
            "airtime": "",
            "airstamp": "2021-08-06T12:00:00+00:00",
            "runtime": 36,
            "rating": {"average": 8},
            "image": {
                "medium": "https://static.tvmaze.com/uploads/images/medium_landscape/344/862256.jpg",
                "original": "https://static.tvmaze.com/uploads/images/original_untouched/344/862256.jpg",
            },
            "summary": "<p>Rebecca hasa special visitor shadow her at work. A player's return is not welcomed by the team.</p>",
            "_links": {
                "self": {"href": "https://api.tvmaze.com/episodes/2138281"},
                "show": {
                    "href": "https://api.tvmaze.com/shows/44458",
                    "name": "Ted Lasso",
                },
            },
        },
        {
            "id": 2142664,
            "url": "https://www.tvmaze.com/episodes/2142664/ted-lasso-2x04-carol-of-the-bells",
            "name": "Carol of the Bells",
            "season": 2,
            "number": 4,
            "type": "regular",
            "airdate": "2021-08-13",
            "airtime": "",
            "airstamp": "2021-08-13T12:00:00+00:00",
            "runtime": 30,
            "rating": {"average": 8.5},
            "image": {
                "medium": "https://static.tvmaze.com/uploads/images/medium_landscape/346/866045.jpg",
                "original": "https://static.tvmaze.com/uploads/images/original_untouched/346/866045.jpg",
            },
            "summary": "<p>It's Christmas in Richmond. Rebecca enlists Ted for a secret mission, Roy and Keeley search for a miracle, and the Higginses open up their home.</p>",
            "_links": {
                "self": {"href": "https://api.tvmaze.com/episodes/2142664"},
                "show": {
                    "href": "https://api.tvmaze.com/shows/44458",
                    "name": "Ted Lasso",
                },
            },
        },
        {
            "id": 2147114,
            "url": "https://www.tvmaze.com/episodes/2147114/ted-lasso-2x05-rainbow",
            "name": "Rainbow",
            "season": 2,
            "number": 5,
            "type": "regular",
            "airdate": "2021-08-20",
            "airtime": "",
            "airstamp": "2021-08-20T12:00:00+00:00",
            "runtime": 38,
            "rating": {"average": 8.3},
            "image": {
                "medium": "https://static.tvmaze.com/uploads/images/medium_landscape/348/871312.jpg",
                "original": "https://static.tvmaze.com/uploads/images/original_untouched/348/871312.jpg",
            },
            "summary": "<p>Nate learns how to be assertive from Keeley and Rebecca. Ted asks Roy for a favor.</p>",
            "_links": {
                "self": {"href": "https://api.tvmaze.com/episodes/2147114"},
                "show": {
                    "href": "https://api.tvmaze.com/shows/44458",
                    "name": "Ted Lasso",
                },
            },
        },
        {
            "id": 2151080,
            "url": "https://www.tvmaze.com/episodes/2151080/ted-lasso-2x06-the-signal",
            "name": "The Signal",
            "season": 2,
            "number": 6,
            "type": "regular",
            "airdate": "2021-08-27",
            "airtime": "",
            "airstamp": "2021-08-27T12:00:00+00:00",
            "runtime": 35,
            "rating": {"average": 8},
            "image": {
                "medium": "https://static.tvmaze.com/uploads/images/medium_landscape/350/875535.jpg",
                "original": "https://static.tvmaze.com/uploads/images/original_untouched/350/875535.jpg",
            },
            "summary": "<p>Ted is fired up that the new team dynamic seems to be working. But will they have a chance in the semifinal?</p>",
            "_links": {
                "self": {"href": "https://api.tvmaze.com/episodes/2151080"},
                "show": {
                    "href": "https://api.tvmaze.com/shows/44458",
                    "name": "Ted Lasso",
                },
            },
        },
        {
            "id": 2156168,
            "url": "https://www.tvmaze.com/episodes/2156168/ted-lasso-2x07-headspace",
            "name": "Headspace",
            "season": 2,
            "number": 7,
            "type": "regular",
            "airdate": "2021-09-03",
            "airtime": "",
            "airstamp": "2021-09-03T12:00:00+00:00",
            "runtime": 35,
            "rating": {"average": 7.6},
            "image": {
                "medium": "https://static.tvmaze.com/uploads/images/medium_landscape/352/880559.jpg",
                "original": "https://static.tvmaze.com/uploads/images/original_untouched/352/880559.jpg",
            },
            "summary": "<p>With things turning around for Richmond, it's time for everyone to work on their issues—like Ted's discomfort, Nate's confidence, and Roy's attention.</p>",
            "_links": {
                "self": {"href": "https://api.tvmaze.com/episodes/2156168"},
                "show": {
                    "href": "https://api.tvmaze.com/shows/44458",
                    "name": "Ted Lasso",
                },
            },
        },
        {
            "id": 2161174,
            "url": "https://www.tvmaze.com/episodes/2161174/ted-lasso-2x08-man-city",
            "name": "Man City",
            "season": 2,
            "number": 8,
            "type": "regular",
            "airdate": "2021-09-10",
            "airtime": "",
            "airstamp": "2021-09-10T12:00:00+00:00",
            "runtime": 45,
            "rating": {"average": 8.1},
            "image": {
                "medium": "https://static.tvmaze.com/uploads/images/medium_landscape/353/884734.jpg",
                "original": "https://static.tvmaze.com/uploads/images/original_untouched/353/884734.jpg",
            },
            "summary": "<p>Ted and Dr. Sharon realize they might have to meet each other halfway. Tensions are high as the team prepares for the semifinal.</p>",
            "_links": {
                "self": {"href": "https://api.tvmaze.com/episodes/2161174"},
                "show": {
                    "href": "https://api.tvmaze.com/shows/44458",
                    "name": "Ted Lasso",
                },
            },
        },
        {
            "id": 2166305,
            "url": "https://www.tvmaze.com/episodes/2166305/ted-lasso-2x09-beard-after-hours",
            "name": "Beard After Hours",
            "season": 2,
            "number": 9,
            "type": "regular",
            "airdate": "2021-09-17",
            "airtime": "",
            "airstamp": "2021-09-17T12:00:00+00:00",
            "runtime": 43,
            "rating": {"average": 6.9},
            "image": {
                "medium": "https://static.tvmaze.com/uploads/images/medium_landscape/356/890404.jpg",
                "original": "https://static.tvmaze.com/uploads/images/original_untouched/356/890404.jpg",
            },
            "summary": "<p>After the semifinal, Beard sets out on an all-night odyssey through London in an effort to collect his thoughts.</p>",
            "_links": {
                "self": {"href": "https://api.tvmaze.com/episodes/2166305"},
                "show": {
                    "href": "https://api.tvmaze.com/shows/44458",
                    "name": "Ted Lasso",
                },
            },
        },
        {
            "id": 2172163,
            "url": "https://www.tvmaze.com/episodes/2172163/ted-lasso-2x10-no-weddings-and-a-funeral",
            "name": "No Weddings and a Funeral",
            "season": 2,
            "number": 10,
            "type": "regular",
            "airdate": "2021-09-24",
            "airtime": "",
            "airstamp": "2021-09-24T12:00:00+00:00",
            "runtime": 46,
            "rating": {"average": 7.6},
            "image": {
                "medium": "https://static.tvmaze.com/uploads/images/medium_landscape/357/894799.jpg",
                "original": "https://static.tvmaze.com/uploads/images/original_untouched/357/894799.jpg",
            },
            "summary": "<p>Rebecca is stunnedby a sudden loss. The team rallies to show their support, but Ted finds himself grappling with a piece of his past.</p>",
            "_links": {
                "self": {"href": "https://api.tvmaze.com/episodes/2172163"},
                "show": {
                    "href": "https://api.tvmaze.com/shows/44458",
                    "name": "Ted Lasso",
                },
            },
        },
        {
            "id": 2172941,
            "url": "https://www.tvmaze.com/episodes/2172941/ted-lasso-2x11-midnight-train-to-royston",
            "name": "Midnight Train to Royston",
            "season": 2,
            "number": 11,
            "type": "regular",
            "airdate": "2021-10-01",
            "airtime": "",
            "airstamp": "2021-10-01T12:00:00+00:00",
            "runtime": 42,
            "rating": {"average": 7.8},
            "image": {
                "medium": "https://static.tvmaze.com/uploads/images/medium_landscape/360/900284.jpg",
                "original": "https://static.tvmaze.com/uploads/images/original_untouched/360/900284.jpg",
            },
            "summary": "<p>A billionaire football enthusiast from Ghana makes Sam an unbelievable offer. Ted plans something special for Dr. Sharon's last day with the team.</p>",
            "_links": {
                "self": {"href": "https://api.tvmaze.com/episodes/2172941"},
                "show": {
                    "href": "https://api.tvmaze.com/shows/44458",
                    "name": "Ted Lasso",
                },
            },
        },
        {
            "id": 2172942,
            "url": "https://www.tvmaze.com/episodes/2172942/ted-lasso-2x12-inverting-the-pyramid-of-success",
            "name": "Inverting the Pyramid of Success",
            "season": 2,
            "number": 12,
            "type": "regular",
            "airdate": "2021-10-08",
            "airtime": "",
            "airstamp": "2021-10-08T12:00:00+00:00",
            "runtime": 49,
            "rating": {"average": 8},
            "image": {
                "medium": "https://static.tvmaze.com/uploads/images/medium_landscape/362/905367.jpg",
                "original": "https://static.tvmaze.com/uploads/images/original_untouched/362/905367.jpg",
            },
            "summary": "<p>Richmond gets their final chance to win promotion as Ted deals with the fallout of Trent Crimm's painfully honest exposé.</p>",
            "_links": {
                "self": {"href": "https://api.tvmaze.com/episodes/2172942"},
                "show": {
                    "href": "https://api.tvmaze.com/shows/44458",
                    "name": "Ted Lasso",
                },
            },
        },
    ]


@pytest.fixture
def shows_episodebynumber_ted_lasso_s1_e3() -> dict:
    """Return the JSON results of https://api.tvmaze.com/shows/44458/episodebynumber?season=1&number=3"""

    return {
        "id": 1914172,
        "url": "https://www.tvmaze.com/episodes/1914172/ted-lasso-1x03-trent-crimm-the-independent",
        "name": "Trent Crimm: The Independent",
        "season": 1,
        "number": 3,
        "type": "regular",
        "airdate": "2020-08-14",
        "airtime": "",
        "airstamp": "2020-08-14T12:00:00+00:00",
        "runtime": 30,
        "rating": {"average": 8.2},
        "image": {
            "medium": "https://static.tvmaze.com/uploads/images/medium_landscape/342/856399.jpg",
            "original": "https://static.tvmaze.com/uploads/images/original_untouched/342/856399.jpg",
        },
        "summary": "<p>To arrange an in-depth exposé, Rebecca pairs cynical journalist Trent Crimm with Ted for a day. Ted and Roy venture into the community.</p>",
        "_links": {
            "self": {"href": "https://api.tvmaze.com/episodes/1914172"},
            "show": {"href": "https://api.tvmaze.com/shows/44458", "name": "Ted Lasso"},
        },
    }


@pytest.fixture
def shows_seasons_ted_lasso() -> list[dict]:
    """Return JSON results of https://api.tvmaze.com/shows/44458/seasons"""

    return [
        {
            "id": 101583,
            "url": "https://www.tvmaze.com/seasons/101583/ted-lasso-season-1",
            "number": 1,
            "name": "",
            "episodeOrder": 10,
            "premiereDate": "2020-08-14",
            "endDate": "2020-10-02",
            "network": None,
            "webChannel": {
                "id": 310,
                "name": "Apple TV+",
                "country": None,
                "officialSite": "https://tv.apple.com/",
            },
            "image": {
                "medium": "https://static.tvmaze.com/uploads/images/medium_portrait/345/862940.jpg",
                "original": "https://static.tvmaze.com/uploads/images/original_untouched/345/862940.jpg",
            },
            "summary": "<p>Jason Sudeikis plays Ted Lasso, a small-time college football coach from Kansas hired to coach a professional soccer team in England, despite having no experience coaching soccer.</p>",
            "_links": {"self": {"href": "https://api.tvmaze.com/seasons/101583"}},
        },
        {
            "id": 112939,
            "url": "https://www.tvmaze.com/seasons/112939/ted-lasso-season-2",
            "number": 2,
            "name": "",
            "episodeOrder": 12,
            "premiereDate": "2021-07-23",
            "endDate": "2021-10-08",
            "network": None,
            "webChannel": {
                "id": 310,
                "name": "Apple TV+",
                "country": None,
                "officialSite": "https://tv.apple.com/",
            },
            "image": {
                "medium": "https://static.tvmaze.com/uploads/images/medium_portrait/389/973653.jpg",
                "original": "https://static.tvmaze.com/uploads/images/original_untouched/389/973653.jpg",
            },
            "summary": "<p>Golden Globe® winner Jason Sudeikis is Ted Lasso, an American football coach hired to manage a British soccer team—despite having no experience. But what he lacks in knowledge, he makes up for with optimism, underdog determination...and biscuits.</p>",
            "_links": {"self": {"href": "https://api.tvmaze.com/seasons/112939"}},
        },
        {
            "id": 116253,
            "url": "https://www.tvmaze.com/seasons/116253/ted-lasso-season-3",
            "number": 3,
            "name": "",
            "episodeOrder": 12,
            "premiereDate": "2023-03-15",
            "endDate": "2023-05-31",
            "network": None,
            "webChannel": {
                "id": 310,
                "name": "Apple TV+",
                "country": None,
                "officialSite": "https://tv.apple.com/",
            },
            "image": {
                "medium": "https://static.tvmaze.com/uploads/images/medium_portrait/447/1118134.jpg",
                "original": "https://static.tvmaze.com/uploads/images/original_untouched/447/1118134.jpg",
            },
            "summary": '<p>In this third seasonof "Ted Lasso," the newly-promoted AFC Richmond faces ridicule as media predictions widely peg them as last in the Premier League and Nate, now hailed as the "wonder kid," has gone to work for Rupert at West Ham United. In the wake of Nate\'s contentious departure from Richmond, Roy Kent steps up as assistant coach, alongside Beard. Meanwhile, while Ted deals with pressures at work, he continues to wrestle with his own personal issues back home, Rebecca is focused on defeating Rupert and Keeleynavigates being the boss of her own PR agency. Things seem to be falling apart both on and off the pitch, but Team Lasso is set to give it their best shot anyway.</p>',
            "_links": {"self": {"href": "https://api.tvmaze.com/seasons/116253"}},
        },
        {
            "id": 174064,
            "url": "https://www.tvmaze.com/seasons/174064/ted-lasso-season-4",
            "number": 4,
            "name": "",
            "episodeOrder": None,
            "premiereDate": None,
            "endDate": None,
            "network": None,
            "webChannel": {
                "id": 310,
                "name": "Apple TV+",
                "country": None,
                "officialSite": "https://tv.apple.com/",
            },
            "image": None,
            "summary": None,
            "_links": {"self": {"href": "https://api.tvmaze.com/seasons/174064"}},
        },
    ]


@pytest.fixture
def singlesearch_shows_ted_lasso() -> dict:
    """Return the JSON results of https://api.tvmaze.com/singlesearch/shows?q=Ted+Lasso"""

    return {
        "id": 44458,
        "url": "https://www.tvmaze.com/shows/44458/ted-lasso",
        "name": "Ted Lasso",
        "type": "Scripted",
        "language": "English",
        "genres": ["Drama", "Comedy", "Sports"],
        "status": "Running",
        "runtime": None,
        "averageRuntime": 42,
        "premiered": "2020-08-14",
        "ended": None,
        "officialSite": "https://tv.apple.com/show/ted-lasso/umc.cmc.vtoh0mn0xn7t3c643xqonfzy",
        "schedule": {"time": "", "days": ["Wednesday"]},
        "rating": {"average": 8.1},
        "weight": 96,
        "network": None,
        "webChannel": {
            "id": 310,
            "name": "Apple TV+",
            "country": None,
            "officialSite": "https://tv.apple.com/",
        },
        "dvdCountry": None,
        "externals": {"tvrage": None, "thetvdb": 383203, "imdb": "tt10986410"},
        "image": {
            "medium": "https://static.tvmaze.com/uploads/images/medium_portrait/457/1142533.jpg",
            "original": "https://static.tvmaze.com/uploads/images/original_untouched/457/1142533.jpg",
        },
        "summary": "<p><b>Ted Lasso </b>centers on an idealistic — and clueless — all-American football coach hired to manage an English football club — despite having no soccer coaching experience at all.</p>",
        "updated": 1724535610,
        "_links": {
            "self": {"href": "https://api.tvmaze.com/shows/44458"},
            "previousepisode": {
                "href": "https://api.tvmaze.com/episodes/2490079",
                "name": "So Long, Farewell",
            },
        },
    }


def test_get_series_id_local(
    fetcher: nielsen.fetcher.TVMaze, ted_lasso_series_id: int, mocker: MockerFixture
) -> None:
    """Get series ID from local config file."""

    # Use a Spy to assert the right function was called by get_series_id.
    id_local = mocker.spy(nielsen.fetcher.TVMaze, "get_series_id_local")

    assert (
        fetcher.get_series_id("Ted Lasso") == ted_lasso_series_id
    ), "Should get ID from config file"

    id_local.assert_called_with(fetcher, "Ted Lasso")


def test_get_series_id_remote_single(
    config: ConfigParser,
    fetcher: nielsen.fetcher.TVMaze,
    ted_lasso_series_id: int,
    mock_get: MockType,
    response_factory: MockType,
    singlesearch_shows_ted_lasso: dict,
    mocker: MockerFixture,
) -> None:
    """Get series ID from TVMaze API with single result."""

    # Clear the config to ensure the series isn't found locally
    config.clear()

    # Use a Spy to assert the right function was called by get_series_id.
    id_singlesearch = mocker.spy(nielsen.fetcher.TVMaze, "get_series_id_singlesearch")

    # Load test fixture with actual API results
    resp_ok: MockType = response_factory(
        "https://api.tvmaze.com/singlesearch/shows?q=Ted+Lasso",
        True,
        singlesearch_shows_ted_lasso,
    )

    mock_get.return_value = resp_ok
    assert (
        fetcher.get_series_id("Ted Lasso") == ted_lasso_series_id
    ), "Should get ID from TVMaze response"

    id_singlesearch.assert_called_once_with(fetcher, "Ted Lasso")
    id_singlesearch.reset_mock()

    # Ensure a bad response returns a 0 for the Series ID
    resp_not_ok: MockType = response_factory(
        "https://api.tvmaze.com/singlesearch/shows?q=useless+search+string", False, {}
    )

    mock_get.return_value = resp_not_ok

    assert (
        fetcher.get_series_id_singlesearch("Useless Search String") == 0
    ), "Should return 0 on a 'not ok' TVMaze response"
    id_singlesearch.assert_called_once_with(fetcher, "Useless Search String")


@pytest.mark.parametrize(
    "series_name, series_id, ok, fixt_name",
    [
        pytest.param(
            "Agents of SHIELD",
            31,
            True,
            "search_shows_agents_of_shield",
            id="Agents of SHIELD",
        ),
        pytest.param(
            "Ted Lasso",
            44458,
            True,
            "search_shows_ted_lasso",
            id="Ted Lasso",
        ),
        pytest.param(
            "Useless Search String",
            0,
            False,
            "search_shows_useless",
            id="Useless Search String",
        ),
    ],
)
def test_get_series_id_remote_multiple(
    config: ConfigParser,
    fetcher: nielsen.fetcher.TVMaze,
    mock_get: MockType,
    mock_input: MockType,
    mock_tv: MockType,
    mocker: MockerFixture,
    ok: bool,
    request: pytest.FixtureRequest,
    response_factory: MockType,
    series_id: int,
    series_name: str,
    fixt_name: str,
) -> None:
    """Get series ID from TVMaze API with multiple results. This test ensures that the
    correct method for getting the series ID was called."""

    # Clear the config to ensure the series isn't found locally.
    config.clear()

    # Use a Spy to assert the right function was called by get_series_id.
    id_search = mocker.spy(nielsen.fetcher.TVMaze, "get_series_id_search")

    # Setup the TV instance
    mock_tv.series = series_name

    # Mock the Response with actual API results.
    resp: MockType = response_factory("", ok, request.getfixturevalue(fixt_name))
    mock_get.return_value = resp

    # Always choose the first result
    mock_input.return_value = "1"

    assert (
        fetcher.get_series_id(series=mock_tv.series, interactive=True) == series_id
    ), "Should get ID from TVMaze response"

    id_search.assert_called_with(fetcher, mock_tv.series)


def test_get_episode_title(
    fetcher: nielsen.fetcher.TVMaze,
    mock_get: MockType,
    response_factory: MockType,
    shows_episodebynumber_ted_lasso_s1_e3: dict,
    mocker: MockerFixture,
) -> None:
    """Get the episode title for a given series, season, and episode number."""

    tv: MockType = mocker.MagicMock(spec=nielsen.media.TV)
    tv.series = "Ted Lasso"
    tv.season = 1
    tv.episode = 3
    title: str = "Trent Crimm: The Independent"

    resp: MockType = response_factory(
        shows_episodebynumber_ted_lasso_s1_e3["url"],
        True,
        shows_episodebynumber_ted_lasso_s1_e3,
    )

    mock_get.return_value = resp
    assert fetcher.get_episode_title(tv) == title


def test_get_episode_title_errors(
    fetcher: nielsen.fetcher.TVMaze, missing_file: pathlib.Path, mocker: MockerFixture
) -> None:
    """Raise errors when insufficient information to search for episode titles."""

    mock_get_series_id: MockType = mocker.patch("nielsen.fetcher.TVMaze.get_series_id")
    mock_get_series_id.return_value = None

    empty: nielsen.media.TV = nielsen.media.TV(missing_file)
    with pytest.raises(ValueError):
        fetcher.get_episode_title(empty)

    mock_get_series_id.return_value = 42
    with pytest.raises(ValueError):
        fetcher.get_episode_title(empty)

    empty.season = 1
    with pytest.raises(ValueError):
        fetcher.get_episode_title(empty)


def test_set_series_id(fetcher: nielsen.fetcher.TVMaze, config: ConfigParser) -> None:
    """Create a mapping between a series name and a TVMaze series ID."""

    series: str = "Foo: The Series"
    id: int = 42

    # Clear the config to ensure things work properly even when there is no section
    # for TVMaze IDs.
    config.clear()

    assert not config.has_option(
        fetcher.IDS, series
    ), "The option should not exist before setting."

    fetcher.set_series_id(series, id)
    assert config.has_option(
        fetcher.IDS, series
    ), "The section and option should both exist after setting."


def test_fetch(
    fetcher: nielsen.fetcher.TVMaze,
    mock_get: MockType,
    mocker: MockerFixture,
    response_factory: MockType,
    shows_episodebynumber_ted_lasso_s1_e3: dict,
) -> None:
    """Fetch and update metadata using information from the given `Media` object."""

    tv: MockType = mocker.MagicMock(spec=nielsen.media.TV)
    tv.title = ""

    tv.series = "Ted Lasso"
    tv.season = 1
    tv.episode = 3

    mock_get.return_value = response_factory(
        shows_episodebynumber_ted_lasso_s1_e3["url"],
        True,
        shows_episodebynumber_ted_lasso_s1_e3,
    )

    assert tv.title == ""
    fetcher.fetch(tv)
    assert (
        tv.title == "Trent Crimm: The Independent"
    ), "Title should be correctly set after fetching."


def test_fetch_no_series_id(
    fetcher: nielsen.fetcher.TVMaze, mocker: MockerFixture
) -> None:
    """Fetch and update metadata using information from the given `Media` object."""

    mock_series_id: MockType = mocker.patch("nielsen.fetcher.TVMaze.get_series_id")
    mock_series_id.return_value = 0
    tv: MockType = mocker.MagicMock(spec=nielsen.media.TV)
    tv.series = ""

    with pytest.raises(ValueError):
        fetcher.fetch(tv)


def test_get_season_id(
    fetcher: nielsen.fetcher.TVMaze,
    mock_get: MockType,
    response_factory: MockType,
    shows_seasons_ted_lasso: list[dict],
    ted_lasso_season2_id: int,
    ted_lasso_series_id: int,
) -> None:
    """Verify the GET request and response handling."""

    mock_get.return_value = response_factory(
        f"https://api.tvmaze.com/shows/{ted_lasso_series_id}/seasons",
        True,
        shows_seasons_ted_lasso,
    )

    season_id: int = fetcher.get_season_id(ted_lasso_series_id, 2)
    mock_get.assert_called_with(
        f"{fetcher.SERVICE}/shows/{ted_lasso_series_id}/seasons"
    )

    assert season_id == ted_lasso_season2_id


def test_get_season_id_no_match(
    fetcher: nielsen.fetcher.TVMaze, mock_get: MockType, ted_lasso_series_id: int
) -> None:
    """Verify the GET request and response handling."""

    mock_get.return_value.json.return_value = {}
    season_id: int = fetcher.get_season_id(ted_lasso_series_id, 0)

    assert season_id == 0


def test_episodebynumber(
    fetcher: nielsen.fetcher.TVMaze,
    mock_get: MockType,
    response_factory: MockType,
    shows_episodebynumber_ted_lasso_s1_e3: dict,
    ted_lasso_series_id: int,
) -> None:
    """Verify the GET request and response handling."""

    mock_get.return_value = response_factory(
        shows_episodebynumber_ted_lasso_s1_e3["url"],
        True,
        shows_episodebynumber_ted_lasso_s1_e3,
    )

    fetcher.episodebynumber("Ted Lasso", 1, 3)
    mock_get.assert_called_with(
        f"{fetcher.SERVICE}/shows/{ted_lasso_series_id}/episodebynumber?season=1&number=3"
    )


def test_episodebynumber_no_series_id(fetcher: nielsen.fetcher.TVMaze) -> None:
    """Raise a ValueError if the series ID is invalid."""

    with pytest.raises(ValueError):
        fetcher.episodebynumber(0, 0, 0)


def test_seasons_episodes(
    fetcher: nielsen.fetcher.TVMaze,
    mock_get: MockType,
    response_factory: MockType,
    seasons_episodes_ted_lasso_s2: list[dict],
    ted_lasso_season2_id: int,
) -> None:
    """Verify the GET request and response handling."""

    mock_get.return_value = response_factory(
        f"{fetcher.SERVICE}/seasons/{ted_lasso_season2_id}/episodes",
        True,
        seasons_episodes_ted_lasso_s2,
    )

    fetcher.seasons_episodes(ted_lasso_season2_id)
    mock_get.assert_called_with(
        f"{fetcher.SERVICE}/seasons/{ted_lasso_season2_id}/episodes"
    )


def test_shows(
    fetcher: nielsen.fetcher.TVMaze, ted_lasso_series_id: int, mock_get: MockType
) -> None:
    """Verify the GET request and response handling."""

    fetcher.shows(ted_lasso_series_id)
    mock_get.assert_called_with(f"{fetcher.SERVICE}/shows/{ted_lasso_series_id}")


def test_pick_series_network() -> None:
    pass


def test_pick_series_streaming() -> None:
    pass


def test_pick_series_minimal(
    fetcher: nielsen.fetcher.TVMaze, mocker: MockerFixture
) -> None:
    """Pick a series with minimal information. Unlikely to happen, but should
    account for incomplete information returned from TVMaze."""

    mock_input: MockType = mocker.patch("builtins.input")

    # Mock a search result with incomplete data
    minimal_result: dict[str, Any] = {
        "show": {
            "name": "Unit Test",
            "id": 12345,
            "premiered": "2024-08-30",
        }
    }

    # Select the only result
    mock_input.return_value = 1

    assert fetcher.pick_series("Unit Test", [minimal_result]) == minimal_result


def test_pick_series_default(
    fetcher: nielsen.fetcher.TVMaze, mocker: MockerFixture
) -> None:
    """Pick a series with unparseable information."""

    mock_input: MockType = mocker.patch("builtins.input")
    unparseable_result: dict[str, Any] = {
        "Invalid": "Form",
    }

    # Select the only result
    mock_input.return_value = 1

    assert (
        fetcher.pick_series("Invalid Results", [unparseable_result])
        == unparseable_result
    )


def test_pretty_series() -> None:
    """Return a nicely formatted series for the picker."""

    # Sample return from the /shows/:id endpoint
    # https://api.tvmaze.com/shows/44458
    data: dict[Any, Any] = {
        "id": 44458,
        "url": "https://www.tvmaze.com/shows/44458/ted-lasso",
        "name": "Ted Lasso",
        "type": "Scripted",
        "language": "English",
        "genres": ["Drama", "Comedy", "Sports"],
        "status": "Running",
        "runtime": None,
        "averageRuntime": 42,
        "premiered": "2020-08-14",
        "ended": None,
        "officialSite": "https://tv.apple.com/show/ted-lasso/umc.cmc.vtoh0mn0xn7t3c643xqonfzy",
        "schedule": {"time": "", "days": ["Wednesday"]},
        "rating": {"average": 8.1},
        "weight": 99,
        "network": None,
        "webChannel": {
            "id": 310,
            "name": "Apple TV+",
            "country": None,
            "officialSite": "https://tv.apple.com/",
        },
        "dvdCountry": None,
        "externals": {"tvrage": None, "thetvdb": 383203, "imdb": "tt10986410"},
        "image": {
            "medium": "https://static.tvmaze.com/uploads/images/medium_portrait/457/1142533.jpg",
            "original": "https://static.tvmaze.com/uploads/images/original_untouched/457/1142533.jpg",
        },
        "summary": "<p><b>Ted Lasso </b>centers on an idealistic — and clueless — all-American football coach hired to manage an English football club — despite having no soccer coaching experience at all.</p>",
        "updated": 1724535610,
        "_links": {
            "self": {"href": "https://api.tvmaze.com/shows/44458"},
            "previousepisode": {
                "href": "https://api.tvmaze.com/episodes/2490079",
                "name": "So Long, Farewell",
            },
        },
    }

    assert nielsen.fetcher.TVMaze.pretty_series(data) == (
        "Ted Lasso - ID: 44458 - https://www.tvmaze.com/shows/44458/ted-lasso\n"
        "Premiered: 2020-08-14 - Status: Running\n"
        "Ted Lasso centers on an idealistic — and clueless — all-American football coach hired to manage an English football club — despite having no soccer coaching experience at all."
    )


def test_pretty_season() -> None:
    """Return a nicely formatted season for the picker."""

    # Sample output from the /seasons/:id/episodes endpoint
    # https://api.tvmaze.com/seasons/112939/episodes
    data: list[dict[Any, Any]] = [
        {
            "id": 2075166,
            "url": "https://www.tvmaze.com/episodes/2075166/ted-lasso-2x01-goodbye-earl",
            "name": "Goodbye Earl",
            "season": 2,
            "number": 1,
            "type": "regular",
            "airdate": "2021-07-23",
            "airtime": "",
            "airstamp": "2021-07-23T12:00:00+00:00",
            "runtime": 34,
            "rating": {"average": 7.8},
            "image": {
                "medium": "https://static.tvmaze.com/uploads/images/medium_landscape/341/852868.jpg",
                "original": "https://static.tvmaze.com/uploads/images/original_untouched/341/852868.jpg",
            },
            "summary": "<p>AFC Richmond brings in a sports psychologist to help the team overcome their unprecedented seven game tie-streak.</p>",
            "_links": {
                "self": {"href": "https://api.tvmaze.com/episodes/2075166"},
                "show": {
                    "href": "https://api.tvmaze.com/shows/44458",
                    "name": "Ted Lasso",
                },
            },
        },
        {
            "id": 2132225,
            "url": "https://www.tvmaze.com/episodes/2132225/ted-lasso-2x02-lavender",
            "name": "Lavender",
            "season": 2,
            "number": 2,
            "type": "regular",
            "airdate": "2021-07-30",
            "airtime": "",
            "airstamp": "2021-07-30T12:00:00+00:00",
            "runtime": 33,
            "rating": {"average": 7.9},
            "image": {
                "medium": "https://static.tvmaze.com/uploads/images/medium_landscape/341/852869.jpg",
                "original": "https://static.tvmaze.com/uploads/images/original_untouched/341/852869.jpg",
            },
            "summary": "<p>Ted is surprised by the reappearance of a familiar face. Roy tries out a new gig.</p>",
            "_links": {
                "self": {"href": "https://api.tvmaze.com/episodes/2132225"},
                "show": {
                    "href": "https://api.tvmaze.com/shows/44458",
                    "name": "Ted Lasso",
                },
            },
        },
        {
            "id": 2138281,
            "url": "https://www.tvmaze.com/episodes/2138281/ted-lasso-2x03-do-the-right-est-thing",
            "name": "Do the Right-est Thing",
            "season": 2,
            "number": 3,
            "type": "regular",
            "airdate": "2021-08-06",
            "airtime": "",
            "airstamp": "2021-08-06T12:00:00+00:00",
            "runtime": 36,
            "rating": {"average": 8},
            "image": {
                "medium": "https://static.tvmaze.com/uploads/images/medium_landscape/344/862256.jpg",
                "original": "https://static.tvmaze.com/uploads/images/original_untouched/344/862256.jpg",
            },
            "summary": "<p>Rebecca has a special visitor shadow her at work. A player's return is not welcomed by the team.</p>",
            "_links": {
                "self": {"href": "https://api.tvmaze.com/episodes/2138281"},
                "show": {
                    "href": "https://api.tvmaze.com/shows/44458",
                    "name": "Ted Lasso",
                },
            },
        },
        {
            "id": 2142664,
            "url": "https://www.tvmaze.com/episodes/2142664/ted-lasso-2x04-carol-of-the-bells",
            "name": "Carol of the Bells",
            "season": 2,
            "number": 4,
            "type": "regular",
            "airdate": "2021-08-13",
            "airtime": "",
            "airstamp": "2021-08-13T12:00:00+00:00",
            "runtime": 30,
            "rating": {"average": 8.5},
            "image": {
                "medium": "https://static.tvmaze.com/uploads/images/medium_landscape/346/866045.jpg",
                "original": "https://static.tvmaze.com/uploads/images/original_untouched/346/866045.jpg",
            },
            "summary": "<p>It's Christmas in Richmond. Rebecca enlists Ted for a secret mission, Roy and Keeley search for a miracle, and the Higginses open up their home.</p>",
            "_links": {
                "self": {"href": "https://api.tvmaze.com/episodes/2142664"},
                "show": {
                    "href": "https://api.tvmaze.com/shows/44458",
                    "name": "Ted Lasso",
                },
            },
        },
        {
            "id": 2147114,
            "url": "https://www.tvmaze.com/episodes/2147114/ted-lasso-2x05-rainbow",
            "name": "Rainbow",
            "season": 2,
            "number": 5,
            "type": "regular",
            "airdate": "2021-08-20",
            "airtime": "",
            "airstamp": "2021-08-20T12:00:00+00:00",
            "runtime": 38,
            "rating": {"average": 8.3},
            "image": {
                "medium": "https://static.tvmaze.com/uploads/images/medium_landscape/348/871312.jpg",
                "original": "https://static.tvmaze.com/uploads/images/original_untouched/348/871312.jpg",
            },
            "summary": "<p>Nate learns how to be assertive from Keeley and Rebecca. Ted asks Roy for a favor.</p>",
            "_links": {
                "self": {"href": "https://api.tvmaze.com/episodes/2147114"},
                "show": {
                    "href": "https://api.tvmaze.com/shows/44458",
                    "name": "Ted Lasso",
                },
            },
        },
        {
            "id": 2151080,
            "url": "https://www.tvmaze.com/episodes/2151080/ted-lasso-2x06-the-signal",
            "name": "The Signal",
            "season": 2,
            "number": 6,
            "type": "regular",
            "airdate": "2021-08-27",
            "airtime": "",
            "airstamp": "2021-08-27T12:00:00+00:00",
            "runtime": 35,
            "rating": {"average": 8},
            "image": {
                "medium": "https://static.tvmaze.com/uploads/images/medium_landscape/350/875535.jpg",
                "original": "https://static.tvmaze.com/uploads/images/original_untouched/350/875535.jpg",
            },
            "summary": "<p>Ted is fired up that the new team dynamic seems to be working. But will they have a chance in the semifinal?</p>",
            "_links": {
                "self": {"href": "https://api.tvmaze.com/episodes/2151080"},
                "show": {
                    "href": "https://api.tvmaze.com/shows/44458",
                    "name": "Ted Lasso",
                },
            },
        },
        {
            "id": 2156168,
            "url": "https://www.tvmaze.com/episodes/2156168/ted-lasso-2x07-headspace",
            "name": "Headspace",
            "season": 2,
            "number": 7,
            "type": "regular",
            "airdate": "2021-09-03",
            "airtime": "",
            "airstamp": "2021-09-03T12:00:00+00:00",
            "runtime": 35,
            "rating": {"average": 7.6},
            "image": {
                "medium": "https://static.tvmaze.com/uploads/images/medium_landscape/352/880559.jpg",
                "original": "https://static.tvmaze.com/uploads/images/original_untouched/352/880559.jpg",
            },
            "summary": "<p>With things turning around for Richmond, it's time for everyone to work on their issues—like Ted's discomfort, Nate's confidence, and Roy's attention.</p>",
            "_links": {
                "self": {"href": "https://api.tvmaze.com/episodes/2156168"},
                "show": {
                    "href": "https://api.tvmaze.com/shows/44458",
                    "name": "Ted Lasso",
                },
            },
        },
        {
            "id": 2161174,
            "url": "https://www.tvmaze.com/episodes/2161174/ted-lasso-2x08-man-city",
            "name": "Man City",
            "season": 2,
            "number": 8,
            "type": "regular",
            "airdate": "2021-09-10",
            "airtime": "",
            "airstamp": "2021-09-10T12:00:00+00:00",
            "runtime": 45,
            "rating": {"average": 8.1},
            "image": {
                "medium": "https://static.tvmaze.com/uploads/images/medium_landscape/353/884734.jpg",
                "original": "https://static.tvmaze.com/uploads/images/original_untouched/353/884734.jpg",
            },
            "summary": "<p>Ted and Dr. Sharon realize they might have to meet each other halfway. Tensions are high as the team prepares for the semifinal.</p>",
            "_links": {
                "self": {"href": "https://api.tvmaze.com/episodes/2161174"},
                "show": {
                    "href": "https://api.tvmaze.com/shows/44458",
                    "name": "Ted Lasso",
                },
            },
        },
        {
            "id": 2166305,
            "url": "https://www.tvmaze.com/episodes/2166305/ted-lasso-2x09-beard-after-hours",
            "name": "Beard After Hours",
            "season": 2,
            "number": 9,
            "type": "regular",
            "airdate": "2021-09-17",
            "airtime": "",
            "airstamp": "2021-09-17T12:00:00+00:00",
            "runtime": 43,
            "rating": {"average": 6.9},
            "image": {
                "medium": "https://static.tvmaze.com/uploads/images/medium_landscape/356/890404.jpg",
                "original": "https://static.tvmaze.com/uploads/images/original_untouched/356/890404.jpg",
            },
            "summary": "<p>After the semifinal, Beard sets out on an all-night odyssey through London in an effort to collect his thoughts.</p>",
            "_links": {
                "self": {"href": "https://api.tvmaze.com/episodes/2166305"},
                "show": {
                    "href": "https://api.tvmaze.com/shows/44458",
                    "name": "Ted Lasso",
                },
            },
        },
        {
            "id": 2172163,
            "url": "https://www.tvmaze.com/episodes/2172163/ted-lasso-2x10-no-weddings-and-a-funeral",
            "name": "No Weddings and a Funeral",
            "season": 2,
            "number": 10,
            "type": "regular",
            "airdate": "2021-09-24",
            "airtime": "",
            "airstamp": "2021-09-24T12:00:00+00:00",
            "runtime": 46,
            "rating": {"average": 7.6},
            "image": {
                "medium": "https://static.tvmaze.com/uploads/images/medium_landscape/357/894799.jpg",
                "original": "https://static.tvmaze.com/uploads/images/original_untouched/357/894799.jpg",
            },
            "summary": "<p>Rebecca is stunned by a sudden loss. The team rallies to show their support, but Ted finds himself grappling with a piece of his past.</p>",
            "_links": {
                "self": {"href": "https://api.tvmaze.com/episodes/2172163"},
                "show": {
                    "href": "https://api.tvmaze.com/shows/44458",
                    "name": "Ted Lasso",
                },
            },
        },
        {
            "id": 2172941,
            "url": "https://www.tvmaze.com/episodes/2172941/ted-lasso-2x11-midnight-train-to-royston",
            "name": "Midnight Train to Royston",
            "season": 2,
            "number": 11,
            "type": "regular",
            "airdate": "2021-10-01",
            "airtime": "",
            "airstamp": "2021-10-01T12:00:00+00:00",
            "runtime": 42,
            "rating": {"average": 7.8},
            "image": {
                "medium": "https://static.tvmaze.com/uploads/images/medium_landscape/360/900284.jpg",
                "original": "https://static.tvmaze.com/uploads/images/original_untouched/360/900284.jpg",
            },
            "summary": "<p>A billionaire football enthusiast from Ghana makes Sam an unbelievable offer. Ted plans something special for Dr. Sharon's last day with the team.</p>",
            "_links": {
                "self": {"href": "https://api.tvmaze.com/episodes/2172941"},
                "show": {
                    "href": "https://api.tvmaze.com/shows/44458",
                    "name": "Ted Lasso",
                },
            },
        },
        {
            "id": 2172942,
            "url": "https://www.tvmaze.com/episodes/2172942/ted-lasso-2x12-inverting-the-pyramid-of-success",
            "name": "Inverting the Pyramid of Success",
            "season": 2,
            "number": 12,
            "type": "regular",
            "airdate": "2021-10-08",
            "airtime": "",
            "airstamp": "2021-10-08T12:00:00+00:00",
            "runtime": 49,
            "rating": {"average": 8},
            "image": {
                "medium": "https://static.tvmaze.com/uploads/images/medium_landscape/362/905367.jpg",
                "original": "https://static.tvmaze.com/uploads/images/original_untouched/362/905367.jpg",
            },
            "summary": "<p>Richmond gets their final chance to win promotion as Ted deals with the fallout of Trent Crimm's painfully honest exposé.</p>",
            "_links": {
                "self": {"href": "https://api.tvmaze.com/episodes/2172942"},
                "show": {
                    "href": "https://api.tvmaze.com/shows/44458",
                    "name": "Ted Lasso",
                },
            },
        },
    ]

    assert nielsen.fetcher.TVMaze.pretty_season(data) == (
        "2x1 - Goodbye Earl\n"
        "https://www.tvmaze.com/episodes/2075166/ted-lasso-2x01-goodbye-earl\n"
        "AFC Richmond brings in a sports psychologist to help the team overcome their unprecedented seven game tie-streak.\n\n"
        "2x2 - Lavender\n"
        "https://www.tvmaze.com/episodes/2132225/ted-lasso-2x02-lavender\n"
        "Ted is surprised by the reappearance of a familiar face. Roy tries out a new gig.\n\n"
        "2x3 - Do the Right-est Thing\n"
        "https://www.tvmaze.com/episodes/2138281/ted-lasso-2x03-do-the-right-est-thing\n"
        "Rebecca has a special visitor shadow her at work. A player's return is not welcomed by the team.\n\n"
        "2x4 - Carol of the Bells\n"
        "https://www.tvmaze.com/episodes/2142664/ted-lasso-2x04-carol-of-the-bells\n"
        "It's Christmas in Richmond. Rebecca enlists Ted for a secret mission, Roy and Keeley search for a miracle, and the Higginses open up their home.\n\n"
        "2x5 - Rainbow\n"
        "https://www.tvmaze.com/episodes/2147114/ted-lasso-2x05-rainbow\n"
        "Nate learns how to be assertive from Keeley and Rebecca. Ted asks Roy for a favor.\n\n"
        "2x6 - The Signal\n"
        "https://www.tvmaze.com/episodes/2151080/ted-lasso-2x06-the-signal\n"
        "Ted is fired up that the new team dynamic seems to be working. But will they have a chance in the semifinal?\n\n"
        "2x7 - Headspace\n"
        "https://www.tvmaze.com/episodes/2156168/ted-lasso-2x07-headspace\n"
        "With things turning around for Richmond, it's time for everyone to work on their issues—like Ted's discomfort, Nate's confidence, and Roy's attention.\n\n"
        "2x8 - Man City\n"
        "https://www.tvmaze.com/episodes/2161174/ted-lasso-2x08-man-city\n"
        "Ted and Dr. Sharon realize they might have to meet each other halfway. Tensions are high as the team prepares for the semifinal.\n\n"
        "2x9 - Beard After Hours\n"
        "https://www.tvmaze.com/episodes/2166305/ted-lasso-2x09-beard-after-hours\n"
        "After the semifinal, Beard sets out on an all-night odyssey through London in an effort to collect his thoughts.\n\n"
        "2x10 - No Weddings and a Funeral\n"
        "https://www.tvmaze.com/episodes/2172163/ted-lasso-2x10-no-weddings-and-a-funeral\n"
        "Rebecca is stunned by a sudden loss. The team rallies to show their support, but Ted finds himself grappling with a piece of his past.\n\n"
        "2x11 - Midnight Train to Royston\n"
        "https://www.tvmaze.com/episodes/2172941/ted-lasso-2x11-midnight-train-to-royston\n"
        "A billionaire football enthusiast from Ghana makes Sam an unbelievable offer. Ted plans something special for Dr. Sharon's last day with the team.\n\n"
        "2x12 - Inverting the Pyramid of Success\n"
        "https://www.tvmaze.com/episodes/2172942/ted-lasso-2x12-inverting-the-pyramid-of-success\n"
        "Richmond gets their final chance to win promotion as Ted deals with the fallout of Trent Crimm's painfully honest exposé."
    )


def test_pretty_episode() -> None:
    """Return a nicely formatted episode for the picker."""

    # Sample output from the /shows/:id/episodebynumber?season=:season&number=:number endpoint
    # https://api.tvmaze.com/shows/44458/episodebynumber?season=3&number=4
    data: dict[str, Any] = {
        "id": 2490069,
        "url": "https://www.tvmaze.com/episodes/2490069/ted-lasso-3x04-big-week",
        "name": "Big Week",
        "season": 3,
        "number": 4,
        "type": "regular",
        "airdate": "2023-04-05",
        "airtime": "",
        "airstamp": "2023-04-05T12:00:00+00:00",
        "runtime": 49,
        "rating": {"average": 7.4},
        "image": {
            "medium": "https://static.tvmaze.com/uploads/images/medium_landscape/454/1135656.jpg",
            "original": "https://static.tvmaze.com/uploads/images/original_untouched/454/1135656.jpg",
        },
        "summary": "\u003cp\u003eEveryone's feeling the pressure as Richmond gear up to play West Ham. Ted is reunited with an old friend.\u003c/p\u003e",
        "_links": {
            "self": {"href": "https://api.tvmaze.com/episodes/2490069"},
            "show": {
                "href": "https://api.tvmaze.com/shows/44458",
                "name": "Ted Lasso",
            },
        },
    }

    assert nielsen.fetcher.TVMaze.pretty_episode(data) == (
        "3x4 - Big Week\n"
        "https://www.tvmaze.com/episodes/2490069/ted-lasso-3x04-big-week\n"
        "Everyone's feeling the pressure as Richmond gear up to play West Ham. Ted is reunited with an old friend."
    )



import nielsen.fetcher


def test_strip_tags() -> None:
    """Should strip HTML tags from the given input."""

    summary: str = """\u003cp\u003eJason Sudeikis plays Ted Lasso, a small-time
        college football coach from Kansas hired to coach a professional soccer team
        in England, despite having no experience coaching soccer.\u003c/p\u003e"""

    stripped: str = """Jason Sudeikis plays Ted Lasso, a small-time
        college football coach from Kansas hired to coach a professional soccer team
        in England, despite having no experience coaching soccer."""

    assert nielsen.fetcher.strip_tags(summary) == stripped
